﻿#ifndef __DING_RTC_ENGINE_INTERFACE_H__
#define __DING_RTC_ENGINE_INTERFACE_H__

#include "engine_types.h"
#include "engine_utils.h"
#include "engine_device_manager.h"

namespace ding {
namespace rtc {


/**
 * \defgroup CPP_DingRtcEngine 全部接口
 * DingRtc 接口
 */

/** \defgroup CPP_DingRtcEngineBase 基础接口
 *  \ingroup CPP_DingRtcEngine
 *  DingRtc 基础接口
 */

 /** \defgroup CPP_DingRtcEngineChannel 频道相关接口
 *  \ingroup CPP_DingRtcEngine
 *  DingRtc 频道相关接口
 */

/** \defgroup CPP_DingRtcEnginePub 发布相关接口
 *  \ingroup CPP_DingRtcEngine
 *  DingRtc 发布相关接口
 */

 /** \defgroup CPP_DingRtcEngineSub 订阅相关接口
 *  \ingroup CPP_DingRtcEngine
 *  DingRtc 订阅相关接口
 */

/** \defgroup CPP_DingRtcEngineVideo 视频相关接口
 *  \ingroup CPP_DingRtcEngine
 *  DingRtc 视频相关接口
 */

 /** \defgroup CPP_DingRtcEngineScreenShare 共享视频相关接口
 *  \ingroup CPP_DingRtcEngine
 *  DingRtc 视频相关接口
 */

 /** \defgroup CPP_DingRtcEngineAudio 音频相关接口
 *  \ingroup CPP_DingRtcEngine
 *  DingRtc 音频相关接口
 */

/** \defgroup CPP_DingRtcEngineMediaEngine 媒体引擎相关接口
 *  \ingroup CPP_DingRtcEngine
 *  DingRtc 媒体引擎相关接口
 */

 /** \defgroup CPP_DingRtcEngineLiveBypass 直播旁路相关接口
 *  \ingroup CPP_DingRtcEngine
 *  DingRtc 直播旁路相关接口
 */

/** \defgroup CPP_DingRtcEngineChannelRelay 跨频道连麦相关接口
 *  \ingroup CPP_DingRtcEngine
 *  DingRtc 跨频道连麦相关接口
 */

 /** \defgroup CPP_DingRtcEnginePreview 预览相关接口
 *  \ingroup CPP_DingRtcEngine
 *  DingRtc 预览相关接口
 */

/** \defgroup CPP_DingRtcEngineUserInfo 远端用户查询接口
 *  \ingroup CPP_DingRtcEngine
 *  DingRtc 远端用户查询接口
 */

 /** \defgroup CPP_DingRtcEngineOthers 其他接口
 *  \ingroup CPP_DingRtcEngine
 *  DingRtc 其他接口
 */

/**
* \defgroup CPP_DingRtcDef 关键类型定义
* DingRtc 关键类型定义
*/

/**
* \defgroup CPP_DingRtcCallback 回调接口
* DingRtc 关键类型定义
*/

/**
 * @defgroup CPP_DingRtcCallbackVideoFrameObserver 视频数据回调及监听
 * @ingroup CPP_DingRtcCallback
 * DingRtc 视频数据回调及监听
 * @{
 */

/**
 * @brief Video observer使用
 * 
 */
class DING_RTC_API RtcEngineVideoFrameObserver {
public:
    virtual ~RtcEngineVideoFrameObserver() {}

    /**
     * @ingroup CPP_DingRtcCallbackVideoFrameObserver
     * @since 3.0
     * @brief 视频格式采集偏好。
    */
    virtual RtcEngineVideoPixelFormat GetVideoFormatPreference() = 0;

    /**
     * @ingroup CPP_DingRtcCallbackVideoFrameObserver
     * @since 3.0
     * @brief 本地采集视频数据。
    */
    virtual bool onCaptureVideoFrame(ding::rtc::RtcEngineVideoFrame &frame) = 0;
};

class DING_RTC_API RtcEngineVideoStreamSink {
public:
    RtcEngineVideoStreamSink();
    virtual ~RtcEngineVideoStreamSink();

    // app need implement this callback
    virtual void OnVideoFrame(ding::rtc::RtcEngineVideoFrame &frame) = 0;

    // sdk internally used
    void *GetSinkImpl();
private:
    void *impl_ = 0;
};
/**
 * @}
 */

/**
 * @defgroup CPP_DingRtcCallbackAudioFrameObserver 音频数据回调及监听
 * @ingroup CPP_DingRtcCallback
 * DingRtc 音频数据回调及监听。
 * @{
 */

class DING_RTC_API RtcEngineAudioFrameObserver {
public:
    virtual ~RtcEngineAudioFrameObserver() {}

    /**
     * @brief 本地订阅音频数据回调。
     * @details 远端所有用户混音后待播放的音频数据。
     * @param frame 音频数据，详见{@link RtcEngineAudioFrame}。
     * @since 3.0
    */
    virtual void OnPlaybackAudioFrame(ding::rtc::RtcEngineAudioFrame &frame) = 0;

    /**
     * @brief 本地采集音频数据回调。
     * @param frame 音频数据，详见{@link RtcEngineAudioFrame}。
     * @since 3.0
     */
    virtual void OnCapturedAudioFrame(ding::rtc::RtcEngineAudioFrame &frame) = 0;

    /**
     * @brief 经过3A处理后的数据回调。
     * @param frame 音频数据，详见{@link RtcEngineAudioFrame}。
     * @since 3.0
    */
    virtual void OnProcessCapturedAudioFrame(ding::rtc::RtcEngineAudioFrame &frame) = 0;

    /**
     * @brief 本地推流音频数据回调。
     * @param frame 音频数据，详见{@link RtcEngineAudioFrame}。
     * @since 3.0
     */
    virtual void OnPublishAudioFrame(ding::rtc::RtcEngineAudioFrame &frame) = 0;
};
/**
 * @}
 */


/**
 * @defgroup CPP_DingRtcCallbackEventListener 回调及监听
 * @ingroup CPP_DingRtcCallback
 * DingRtc 回调及监听
 * @{
 */

/**
 * @brief SDK事件回调基础类。
 */
class DING_RTC_API RtcEngineEventListener {
public:
    virtual ~RtcEngineEventListener() {}
    
    /**
     * @ingroup CPP_DingRtcCallbackEventListener
     * @since 3.0
     * @brief 加入频道结果。
     * @details 当应用调用 {@link RtcEngine::JoinChannel} 方法时，该回调表示成功/失败加入频道，并且返回频道加入的相关信息以及加入频道耗时。
     * @param result 加入频道结果，成功返回0，失败返回错误码。
     * @param channel 频道id。
     * @param userId  用户ID。
     * @param elapsed 加入频道耗时。
     */
    virtual void OnJoinChannelResult(int result, const char *channel, const char *userId, int elapsed) {}

    /**
     * @ingroup CPP_DingRtcCallbackEventListener
     * @since 3.0
     * @brief 离开频道结果。
     * @details 应用调用 {@link RtcEngine::LeaveChannel} 方法时，该回调表示成功/失败离开频道，回调将会返回离会的result和该频道的基本信息,如果 {@link RtcEngine::LeaveChannel} 后直接 {@link RtcEngine::Destroy} SDK，将不会收到此回调。
     * @param result 离开频道结果，成功返回0，失败返回错误码。
     * @param stats 本次频道内会话的数据统计汇总。
     */
    virtual void OnLeaveChannelResult(int result, ding::rtc::RtcEngineStats stats) {}
    
    /**
     * @ingroup CPP_DingRtcCallbackEventListener
     * @since 3.0
     * @brief 远端用户加入频道回调。
     * @param uid 用户ID 从App server分配的唯一标示符。
     * @param elapsed 用户加入频道时的耗时。
     */
    virtual void OnRemoteUserOnLineNotify(const char *uid, int elapsed) {}

    /**
     * @ingroup CPP_DingRtcCallbackEventListener
     * @since 3.0
     * @brief 远端用户离开频道回调。
     * @param uid 用户ID 从App server分配的唯一标示符。
     * @param reason 用户离线的原因，详见 {@link RtcEngineUserOfflineReason}。
     */
    virtual void OnRemoteUserOffLineNotify(const char *uid, ding::rtc::RtcEngineUserOfflineReason reason) {}

    /**
     * @ingroup CPP_DingRtcCallbackEventListener
     * @since 3.0
     * @brief 被服务器踢出频道的消息。
     * @param code onBye类型，详见 {@link RtcEngineOnByeType}。
     */
    virtual void OnBye(RtcEngineOnByeType code) {}
    
    /**
     * @ingroup CPP_DingRtcCallbackEventListener
     * @since 3.0
     * @brief 网络连接状态改变的回调。
     * @param status 当前网络链接状态。
     * @param reason 网络链接状态改变原因。
     */
    virtual void OnConnectionStatusChanged(ding::rtc::RtcEngineConnectionStatus status, ding::rtc::RtcEngineConnectionStatusChangeReason reason) {}
    
    /**
     * @ingroup CPP_DingRtcCallbackEventListener
     * @since 3.0
     * @brief 网络质量变化时发出的消息。
     * @param uid  网络质量发生变化的用户uid。
     * @param upQuality 上行网络质量，详见 {@link RtcEngineNetworkQuality}。
     * @param downQuality 下行网络质量，详见 {@link RtcEngineNetworkQuality}。
     * @note 当网络质量发生变化时触发，uid为空时代表用户自己的网络质量发生变化。
     */
    virtual void OnNetworkQualityChanged(const char *uid,
                                         ding::rtc::RtcEngineNetworkQuality upQuality,
                                         ding::rtc::RtcEngineNetworkQuality downQuality) {}

    /**
     * @ingroup CPP_DingRtcCallbackEventListener
     * @since 3.0
     * @brief 埋点log。
     * @param log 埋点统计信息。
     * @note 用处不大，将来版本移除。
     */
    virtual void OnStatisticsLog(String const& log) {};
    
    /**
     * @ingroup CPP_DingRtcCallbackEventListener
     * @since 3.0
     * @brief 如果engine出现warning，通过这个消息通知app。
     * @param warn 警告类型。
     * @param msg 警告信息。
     */
    virtual void OnOccurWarning(int warn, const char *msg) {}

    /**
     * @ingroup CPP_DingRtcCallbackEventListener
     * @since 3.0
     * @brief 如果engine出现error，通过这个消息通知app。
     * @param error  错误类型，参考 {@link RtcEngineErrorCode}。
     * @param msg 错误描述。
     */
    virtual void OnOccurError(int error, const char* msg) {}

    /**
     * @ingroup CPP_DingRtcCallbackEventListener
     * @since 3.0
     * @brief 埋点log。
     * @param level 日志等级。
     * @param log 埋点统计信息。
     * @note 用处不大，将来版本移除。
     */
    virtual void OnStatisticsLevelLog(int level, const String& log) {}

    /**
     * @ingroup CPP_DingRtcCallbackEventListener
     * @since 3.0
     * @brief 质量报告，相对OnLocalAudioStats，OnRemoteVideoStats更加简练，适合在UI上展示。
     * @param quality 采集到的各项指标数据。
     */
    virtual void OnQualityReport(const ding::rtc::QualityReport& quality) {}

    /**
     * @ingroup CPP_DingRtcCallbackEventListener
     * @since 3.0
     * @brief 本地音频统计信息。
     * @param localAudioStats 本地音频统计信息。
     * @note SDK周期触发一次此统计信息回调。
     */
    virtual void OnLocalAudioStats(const ding::rtc::LocalAudioStats& localAudioStats) {}

    /**
     * @ingroup CPP_DingRtcCallbackEventListener
     * @since 3.0
     * @brief 本地视频统计信息。
     * @param localVideoStats 本地视频统计信息。
     * @note SDK周期触发一次此统计信息回调。
     */
    virtual void OnLocalVideoStats(const ding::rtc::LocalVideoStats& localVideoStats) {}

    /**
     * @ingroup CPP_DingRtcCallbackEventListener
     * @since 3.0
     * @brief 远端音频统计信息。
     * @param remoteAudioStats 远端音频统计信息。
     * @note SDK周期触发一次此统计信息回调。
     */
    virtual void OnRemoteAudioStats(const ding::rtc::RemoteAudioStats& remoteAudioStats) {}

    /**
     * @ingroup CPP_DingRtcCallbackEventListener
     * @since 3.0
     * @brief 远端视频统计信息。
     * @param remoteVideoStats 远端视频统计信息。
     * @note SDK周期触发一次此统计信息回调。
     */
    virtual void OnRemoteVideoStats(const ding::rtc::RemoteVideoStats& remoteVideoStats) {}

    /**
     * @ingroup CPP_DingRtcCallbackEventListener
     * @since 3.0
     * @brief 当前会话统计信息回调。
     * @param stats 会话统计信息。
     * @note SDK周期触发一次此统计信息回调。
     */
    virtual void OnStats(const ding::rtc::RtcEngineStats& stats) {}

    /**
     * @ingroup CPP_DingRtcCallbackEventListener
     * @since 3.0
     * @brief 音频推流变更回调。
     * @param oldState 之前的推流状态。
     * @param newState 当前的推流状态。
     * @param elapseSinceLastState 状态变更时间间隔。
     * @param channel 当前频道id。
     */
    virtual void OnAudioPublishStateChanged(ding::rtc::RtcEnginePublishState oldState,
                                            ding::rtc::RtcEnginePublishState newState,
                                            int elapseSinceLastState,
                                            const char* channel){};

    /**
     * @ingroup CPP_DingRtcCallbackEventListener
     * @since 3.0
     * @brief 视频推流变更回调。
     * @param oldState 之前的推流状态。
     * @param newState 当前的推流状态。
     * @param elapseSinceLastState 状态变更时间间隔。
     * @param channel 当前频道id。
     */
    virtual void OnVideoPublishStateChanged(ding::rtc::RtcEnginePublishState oldState,
                                            ding::rtc::RtcEnginePublishState newState,
                                            int elapseSinceLastState,
                                            const char* channel){};

    /**
     * @ingroup CPP_DingRtcCallbackEventListener
     * @since 3.0
     * @brief 屏幕分享推流变更回调。
     * @param oldState 之前的推流状态。
     * @param newState 当前的推流状态。
     * @param elapseSinceLastState 状态变更时间间隔。
     * @param channel 当前频道id。
     */
    virtual void OnScreenSharePublishStateChanged(ding::rtc::RtcEnginePublishState oldState,
                                                  ding::rtc::RtcEnginePublishState newState,
                                                  int elapseSinceLastState,
                                                  const char* channel){};

    /**
     * @ingroup CPP_DingRtcCallbackEventListener
     * @since 3.0
     * @brief 远端用户的音视频流发生变化回调。
     * @details 该回调在以下场景会被触发：
     * - 当远端用户从未推流变更为推流（包括音频和视频）。
     * - 当远端用户从已推流变更为未推流（包括音频和视频）。
     * @param uid userId，从App server分配的唯一标示符。
     * @param audioTrack 音频流类型，详见 {@link RtcEngineAudioTrack}。
     * @param videoTrack 视频流类型，详见 {@link RtcEngineVideoTrack}。
     */
    virtual void OnRemoteTrackAvailableNotify(const char *uid,
                                              ding::rtc::RtcEngineAudioTrack audioTrack,
                                              ding::rtc::RtcEngineVideoTrack videoTrack) {};
    
    /**
     * @ingroup CPP_DingRtcCallbackEventListener
     * @since 3.0
     * @brief 远端用户静音/取消静音回调。
     * @param uid 远端用户ID。
     * @param isMute 
     * - 该用户是否静音。
     * - true：静音。
     * - false：取消静音。
     */
    virtual void OnUserAudioMuted(const char* uid, bool isMute) {}
    
    /**
     * @ingroup CPP_DingRtcCallbackEventListener
     * @since 3.0
     * @brief 对端用户发送视频黑帧数据发送通知。
     * @param uid 执行 muteVideo 的用户。
     * @param isMute 是否推黑帧。
     * @param videoTrack 
     * - 屏幕流或者相机流，参考 {@link RtcEngineVideoTrack}。
     * - true：推流黑帧。
     * - false：正常推流。
     */
    virtual void OnUserVideoMuted(const char* uid, bool isMute, ding::rtc::RtcEngineVideoTrack videoTrack) {};

    /**
     * @ingroup CPP_DingRtcCallbackEventListener
     * @since 3.0
     * @brief 远端用户的第一帧视频帧显示时触发这个消息。
     * @param uid user id。
     * @param videoTrack 屏幕流或者相机流，参考 {@link RtcEngineVideoTrack}。
     * @param width 视频宽度。
     * @param height 视频高度。
     * @param elapsed 本地用户加入频道直至该回调触发的延迟总耗时（毫秒）。
     * @note 该接口用于远端用户的第一帧视频帧显示时的回调。
     */
    virtual void OnFirstRemoteVideoFrameDrawn(const char *uid, ding::rtc::RtcEngineVideoTrack videoTrack, int width, int height, int elapsed) {}

    /**
     * @ingroup CPP_DingRtcCallbackEventListener
     * @since 3.0
     * @brief 预览开始显示第一帧视频帧时触发这个消息。
     * @param width 本地预览视频宽度。
     * @param height 本地预览视频高度。
     * @param elapsed 从本地用户加入频道直至该回调触发的延迟总耗时（毫秒）。
     * @note 该接口用于预览开始显示第一帧视频帧时的回调。
     */
    virtual void OnFirstLocalVideoFrameDrawn(int width, int height, int elapsed) {}

    /**
     * @ingroup CPP_DingRtcCallbackEventListener
     * @since 3.0
     * @brief 视频首包发送回调。
     * @param videoTrack 发送视频track。
     * @param timeCost 耗时（毫秒）。
     * @note 该接口用于视频首包发送的回调。
     */
    virtual void OnFirstVideoPacketSend(ding::rtc::RtcEngineVideoTrack videoTrack, int timeCost) {}

    /**
     * @ingroup CPP_DingRtcCallbackEventListener
     * @since 3.0
     * @brief 视频首包接收回调。
     * @param uid User ID，从App server分配的唯一标示符。
     * @param videoTrack 接收视频track。
     * @param timeCost 耗时（毫秒）。
     * @note 该接口用于视频首包接收的回调。
     */
    virtual void OnFirstVideoPacketReceived(const char* uid,
                                            ding::rtc::RtcEngineVideoTrack videoTrack,
                                            int timeCost) {}

    /**
     * @ingroup CPP_DingRtcCallbackEventListener
     * @since 3.0
     * @brief 收到远端用户视频首帧的回调。
     * @param uid User ID，从App server分配的唯一标示符。
     * @param videoTrack 接收视频track。
     * @param timeCost 耗时（毫秒）。
     * @note 该接口用于收到远端用户视频首帧的回调。
    */
    virtual void OnFirstVideoFrameReceived(const char* uid,
                                        ding::rtc::RtcEngineVideoTrack videoTrack,
                                        int timeCost) {};

    /**
     * @ingroup CPP_DingRtcCallbackEventListener
     * @since 3.0
     * @brief 音频首包发送回调。
     * @details 在首个音频数据包发送出去时触发此回调。
     * @param timeCost 发送耗时，从入会开始到音频首包发送出去的耗时。
     */
    virtual void OnFirstAudioPacketSend(int timeCost) {};

    /**
     * @ingroup CPP_DingRtcCallbackEventListener
     * @since 3.0
     * @brief 音频首包接收回调。
     * @details 在接收到远端首个音频数据包时触发此回调。
     * @param uid 远端用户ID，从App server分配的唯一标识符。
     * @param timeCost 接收耗时，从入会开始到音频首包接收到的耗时。
     */
    virtual void OnFirstAudioPacketReceived(const char* uid, int timeCost) {}

    /**
     * @ingroup CPP_DingRtcCallbackEventListener
     * @since 3.0
     * @brief 音频订阅情况变更回调。
     * @param uid userId，从App server分配的唯一标示符。
     * @param oldState 之前的订阅状态。
     * @param newState 当前的订阅状态。
     * @param elapseSinceLastState 两次状态变更时间间隔（毫秒）。
     * @param channel 当前频道id。
     */
    virtual void OnAudioSubscribeStateChanged(const char *uid,
                                              ding::rtc::RtcEngineSubscribeState oldState,
                                              ding::rtc::RtcEngineSubscribeState newState,
                                              int elapseSinceLastState,
                                              const char *channel) {};

    /**
     * @ingroup CPP_DingRtcCallbackEventListener
     * @since 3.0
     * @brief 相机流订阅情况变更回调。
     * @param uid userId，从App server分配的唯一标示符。
     * @param oldState 之前的订阅状态。
     * @param newState 当前的订阅状态。
     * @param elapseSinceLastState 两次状态变更时间间隔（毫秒）。
     * @param channel 当前频道id。
     */
    virtual void OnVideoSubscribeStateChanged(const char *uid,
                                              ding::rtc::RtcEngineSubscribeState oldState,
                                              ding::rtc::RtcEngineSubscribeState newState,
                                              int elapseSinceLastState,
                                              const char *channel) {};

    /**
     * @ingroup CPP_DingRtcCallbackEventListener
     * @since 3.0
     * @brief 屏幕分享流订阅情况变更回调。
     * @param uid userId，从App server分配的唯一标示符。
     * @param oldState 之前的订阅状态。
     * @param newState 当前的订阅状态。
     * @param elapseSinceLastState 两次状态变更时间间隔（毫秒）。
     * @param channel 当前频道id。
     */
    virtual void OnScreenShareSubscribeStateChanged(const char *uid,
                                                    ding::rtc::RtcEngineSubscribeState oldState, 
                                                    ding::rtc::RtcEngineSubscribeState newState,
                                                    int elapseSinceLastState,
                                                    const char *channel) {};

    /**
     * @ingroup CPP_DingRtcCallbackEventListener
     * @since 3.0
     * @brief 不同档次视频订阅情况变更回调。
     * @param uid userId，从App server分配的唯一标示符。
     * @param oldStreamType 之前的订阅的视频档次。
     * @param newStreamType 当前的订阅的视频档次。
     * @param elapseSinceLastState 视频档次变更时间间隔（毫秒）。
     * @param channel 当前频道id。
     * @note 订闭某个视频档次，并不表示一定能够收到该档次的视频。接收到的档次取決于推流情況以及网络情況。
     */
    virtual void OnSubscribeStreamTypeChanged(const char *uid,
                                                ding::rtc::RtcEngineVideoStreamType oldStreamType, 
                                                ding::rtc::RtcEngineVideoStreamType newStreamType, 
                                                int elapseSinceLastState, 
                                                const char *channel) {};


    /**
     * @ingroup CPP_DingRtcCallbackEventListener
     * @since 3.0
     * @brief 音频采集设备音量回调。
     * @param level 音频采集设备音量值，取值[0, 100]。
     */
    virtual void OnRecordingDeviceAudioLevel(int level) {};

    /**
     * @ingroup CPP_DingRtcCallbackEventListener
     * @since 3.0
     * @brief 音频播放设备音量回调。
     * @param level 音频播放设备音量值，取值[0, 100]。
     */
    virtual void OnPlayoutDeviceAudioLevel(int level) {};

    /**
     * @since 3.0
     * @brief 远端用户音量提示回调。
     * @param speakers 用户音量信息，详见参考{@link AudioVolumeInfo}，如果 speakers 为空，则表示远端用户不发流或没有远端用户。
     * @param speakerNumber 远端用户数量。
     */
    virtual void OnAudioVolumeIndication(const ding::rtc::AudioVolumeInfo* speakers, unsigned int speakerNumber) {};

    /**
     * @ingroup CPP_DingRtcCallbackEventListener
     * @since 3.0
     * @brief 音频设备状态变更。
     * @param deviceId 设备id。
     * @param deviceType  设备类型。
     * @param deviceState 设备状态。
     */
    virtual void OnAudioDeviceStateChanged(const char *deviceId, ding::rtc::RtcEngineDeviceType deviceType, ding::rtc::RtcEngineDeviceState deviceState) {};

#if defined(ALI_OS_ANDROID) || defined(ALI_OS_IOS)
    /**
     * @brief 音频路由发生变化回调。
     * @details 所谓“音频路由”，是指声音是从手机的扬声器还是从听筒中播放出来，音频路由变化也就是声音的播放位置发生了变化。
     * - 当音频路由为听筒时，声音比较小，只有将耳朵凑近才能听清楚，隐私性较好，适合用于接听电话。
     * - 当音频路由为扬声器时，声音比较大，不用将手机贴脸也能听清，因此可以实现“免提”的功能。
     * @param audioRouteType 当前使用的音频路由，详情参考{@link RtcEngineAudioRouteType}
     */
    virtual void OnAudioRouteChanged(ding::rtc::RtcEngineAudioRouteType audioRouteType) {}
#endif

    /**
     * @ingroup CPP_DingRtcCallbackEventListener
     * @since 3.0
     * @brief API 方法已执行回调。
     * @param error 当该方法调用失败时 SDK 返回的错误码。如果该方法调用成功，SDK 将返回 0。失败，非0。
     * @param api SDK 执行的 API。
     * @param result SDK 调用 API 的调用结果。
     */
    virtual void OnApiCalledExecuted(int error, const char *api, const char *result) {};
    
    /**
     * @ingroup CPP_DingRtcCallbackEventListener
     * @since 3.0
     * @brief 视频截图回调。
     * @param userId 用户id，为空或者为""代表本地用户。
     * @param videoTrack 截图视频track，参考 {@link RtcEngineVideoTrack}。
     * @param filePath 截图的本地保存路径。
     * @param width 截图宽度。
     * @param height 截图高度。
     * @param success 截图是否成功。
     * @note 该接口用于截图回调。
     */
    virtual void OnSnapshotComplete(const char* userId, ding::rtc::RtcEngineVideoTrack videoTrack, const char* filePath, int width, int height, bool success) {}
};

/**
 * @}
 */

/** 
 * \addtogroup Engine  引擎参考
*/
//! @{

/**
 * @interface RtcEngine
 * @brief RTC引擎的主要控制类
 */
class DING_RTC_API RtcEngine
{
protected:
  /**
  * @{ @name OverallApi  Engine的生命周期，日志，错误码
  */
    RtcEngine() {}
    virtual ~RtcEngine() {}
  /**
  * @}
  */

public:
  /**
  * @{ @name OverallApi  Engine的生命周期，日志，错误码
  */
    /**
     * @ingroup CPP_DingRtcEngineBase
     * @since 3.0
     * @brief 创建一个RtcEngine实例。
     * @param extras 用于接收来自客户设置的参数，通过JSON配置SDK的特别功能，可以是空字符串。
     * @details 该方法为同步调用，支持多实例。
     */
    static RtcEngine *Create(const char *extras);

    /**
     * @ingroup CPP_DingRtcEngineBase
     * @since 3.0
     * @brief 销毁RtcEngine实例。
     * @details 调用该方法将会释放所有内部使用的资源,当完成音视频通信之后都建议调用该方法
     *          释放实例.调用该方法后，你将不能再使用 {@link RtcEngine} 的其他方法和任何
     *          回调，如需要再次使用需要重新调用 {@link RtcEngine::Create} 方法创建一个
     *          新的实例。
     * @note 该方法为同步调用，需要等待内部资源释放之后才能执行其他方法。为避免主线程阻塞，
     *       建议开发者放在子线程调用该方法。
     * @note 为避免死锁，不建议在任何SDK的回调中调用本方法。
     */
    static void Destroy(RtcEngine *instance);

    /**
     * @brief 查询sdk当前版本号。
     * @return 当前的SDK版本号，格式为字符串，例："2.5.0"。
     * @note 在任何时刻都可以调用。
     */
    static const char *GetSDKVersion();

    /// @private
    static const char *GetSDKBuild();

    /**
     * @ingroup CPP_DingRtcEngineBase
     * @since 3.0
     * @brief 设置SDK日志文件保存路径。
     * @param logDirPath 日志文件保存目录的绝对路径，该参数是一个目录路径，而不是文件的路径。
     * @return 
     * - 0，成功
     * - 非0，失败。其中有两种错误码，释义如下：
     *   - {@link RtcEngineErrorLogRootPathError}：路径已存在或因权限问题不可访问。
     * 
     * @note 调用此接口，请在调用其他所有SDK接口前调用本接口，避免日志出现丢失。同时，App必须保证指定的目录已存在并且可以写入。
     * @note 如果，目标路径不可写入或创建日志文件失败，则会使用默认日志路径。本次设置将无效。
     * @note 默认路径：windows平台，在用户目录下dingding/log。
     */
    static int SetLogDirPath(const char *logDirPath);

    /**
     * @ingroup CPP_DingRtcEngineOthers
     * @since 3.0
     * @brief 获取当前SDK日志文件保存路径，包含SDK添加的子目录。
     * @param buf 用于接收目录，不允许NULL。
     * @param maxBufSize buf的大小。拷贝时包含字符串的结束符'\0'。
     * @return
     * - true, 成功，路径存放在buf中；
     * - false, 失败，一种可能是提供的buf长度过短，放不下；另一种可能是尚未设置日志路径。
     */
    static bool GetLogDirPath(char *buf, int maxBufSize);

    /**
     * @ingroup CPP_DingRtcEngineOthers
     * @since 3.0
     * @brief 设置日志等级。
     * @param logLevel Log级别，详见{@link RtcEngineLogLevel}枚举类型。
     * @return
     * - 0：成功；
     * - 非0：失败。比如没有先调用SetLogDirPath。
     *
     * @note 设置 SDK 的输出日志等级。默认值为 {@link RtcEngineLogLevelInfo}。
     * @note 调用此接口，请在仅次于SetLogDirPath接口调用之后再调用本接口，避免因日志等级切换造成的打印丢失。
     */
    static int SetLogLevel(ding::rtc::RtcEngineLogLevel logLevel);

    /**
     * @ingroup CPP_DingRtcEngineOthers
     * @since 3.0
     * @brief 根据错误码，获取错误码描述，也适用于获取 Warning 的描述。
     * @param error_code 错误码，详见{@link RtcEngineErrorCode}。
     * @return 错误码描述字符串。
     * @note 在任何时刻都可以调用。
     */
    static const char* GetErrorDescription(int error_code);

    /**
     * @ingroup CPP_DingRtcEngineBase
     * @since 3.0
     * @brief 设置相关监听事件回调。
     * @details {@link RtcEngineEventListener}类是SDK向应用发送自身回调事件的通知,
     *          应用开发者需要实现该接口类的方法来获取相关事件通知。默认所有接口都有空实现，
     *          开发者不需要实现所有方法，可根据自身业务来实现对应的事件方法。
     * @param listener  用于接收来自engine的消息。
     * @note 所有的回调方法都是SDK内部的线程,不可以做UI操作，有UI相关操作需要抛主线程操作。
     * @note 不建议在回调方法中做耗时操作,如调用RtcEngine本身的Destroy方法，这将有可能
     *       引起不必要的阻塞从而影响SDK的运行。
     */
    virtual int SetEngineEventListener(RtcEngineEventListener *listener) = 0;

    /**
     * @ingroup CPP_DingRtcEngineAudio
     * @since 3.0
     * @brief 获取音频设备管理类接口。
     * @return 音频设备管理类接口。
     */
    virtual RtcEngineAudioDeviceManager* GetAudioDeviceManager() = 0;


    /**
     * @ingroup CPP_DingRtcEngineVideo
     * @since 3.0
     * @brief 获取视频设备管理类接口。
     * @return 视频设备管理类接口。
     */
    virtual RtcEngineVideoDeviceManager* GetVideoDeviceManager() = 0;

    /**
     * @ingroup CPP_DingRtcEngineOthers
     * @since 3.0
     * @brief 获取当前网络链接状态。
     * @return 当前链接状态，参考{@link RtcEngineConnectionStatus}。
     * @note 网络链接状态发生变化时，SDK会通过{@link RtcEngineEventListener::OnConnectionStatusChanged}回调通知。
     */
    virtual RtcEngineConnectionStatus GetCurrentConnectionStatus() = 0;
  /**
  * @}
  */

  /**
  * @{ @name ChannelApi  频道管理
  */
    /**
     * @ingroup CPP_DingRtcEngineChannel
     * @since 3.0
     * @brief 该方法让用户加入频道，一个频道内可以加入多个用户进行互相通话。
     * @param authInfo    认证信息，从App Server获取。
     * @param userName    任意用于显示的用户名称。不是User ID。
     * @return
     * - 0：成功；
     * - <0：失败。
     * @note
     *    - 异步接口。是否成功入会，通过 {@link RtcEngineEventListener::OnJoinChannelResult}判断。
     *    - 正常情况一个Appid对应一个App，只有同一个AppId的应用可进行互相童话，不同AppId和channelID不同都不能互通。
     *    - 当入会成功后，会触发远端用户收到 {@link RtcEngineEventListener::OnJoinChannelResult} 回调。
     *    - 如果已在会中想重新入会，需要先调用 {@link LeaveChannel} 离会，且确保 {@link RtcEngineEventListener::OnLeaveChannelResult} 回调收到后，再次调用进行入会。
     */
    virtual int JoinChannel(const ding::rtc::RtcEngineAuthInfo &authInfo, const char *userName) = 0;

    /**
     * @ingroup CPP_DingRtcEngineChannel
     * @since 3.0
     * @brief  离开频道，即挂断或者停止通话。
     * @details 如果当前不在频道内，{@link LeaveChannel} 不会有任何影响。{@link LeaveChannel} 会产生消息通知频道内其他用户。
     * @details 调用 {@link JoinChannel}  后必须要调用 {@link LeaveChannel} 离开频道，否则无法进入下一个频道通话。
     * @details 异步方法，成功调用后需要收到 {@link RtcEngineEventListener::OnLeaveChannelResult} 才会真正退出频道，离开频道成功远端用户会触发会收到 {@link RtcEngineEventListener::OnRemoteUserOnLineNotify}回调。
     * @return
     * - 0：成功；
     * - 非0：失败。
     * @note
     *  - 如果调用了 {@link LeaveChannel} 后立刻调用 {@link Destroy} 方法，SDK将会无法触发 {@link RtcEngineEventListener::OnLeaveChannelResult}。
     */
    virtual int LeaveChannel() = 0;

    /**
     * @ingroup CPP_DingRtcEngineUserInfo
     * @since 3.0
     * @brief 获取远端在线用户列表。
     * @param userList 当前频道内所有在线用户userid列表。
     */
    virtual void GetOnlineRemoteUsers(StringArray &userList) = 0;

    /**
     * @ingroup CPP_DingRtcEngineUserInfo
     * @since 3.0
     * @brief 查询远端用户状态。        
     * @details 通过 uid 查询，指定远端用户的状态信息，参数dict类型为标准std::map<std::string， std::string>容器，通过键值对的方式进行访问。 
     * @param uid 用户ID。从App server分配的唯一标示符。
     * @param dict App提供的容器，用于存放用户数据。
     * @return 
     * - 0：成功；
     * - 非0：失败，返回值为具体的错误码。
     * @note Key信息： 
    *  - userID：远端用户的ID。
     *  - isOnline：远端用户是否在线。
     *  - displayName：远端用户的名称。
     *  - hasAudio：远端用户是否推送了音频流。
     *  - hasCamera：远端用户是否推送了相机流。
     *  - hasScreenSharing：远端用户是否推送了屏幕流。
     *  - requestAudio：本端是否订阅了此远端用户的音频流。
     *  - requestCamera：本端是否订阅了此远端用户的相机流。此状态，系在成功订阅该用户摄像头流前的状态。
     *  - requestScreenSharing：本端是否订阅了此远端用户的屏幕流。此状态，系在成功订阅该用户屏幕共享前的状态。
     *  - preferCameraType：如果网络带宽允许，优先订阅的相机流的大小规格，选择范围：FHD, HD, SD, LD。
     *  - subScribedAudio：本端是否拉到了此远端用户的音频流。
     *  - subScribedCameraType：本端拉到了此远端用户的相机流的大小规格，可能的值：FHD, HD, SD, LD。 
     *  - subScribedScreenSharing：本端是否拉到了此远端用户的屏幕流。
     *  - hasCameraView：本端是否设置了此远端用户相机流的view。
     *  - hasScreenView：本端是否设置了此远端用户屏幕流的view。
     *  - isCameraMirror：本端是否设置了此远端用户的相机流为镜像。
     *  - isScreenMirror：本端是否设置了此远端用户的屏幕流为镜像。
     */
    virtual int GetUserInfo(const char *uid, Dictionary &dict) = 0;

    /**
     * @ingroup CPP_DingRtcEngineUserInfo
     * @since 3.0
     * @brief 查询用户是否在线。
     * @param uid 用户ID，从App server分配的唯一标示符。不可为NULL。
     * @return
     *  - true：用户在线；
     *  - false：用户不在线。
     * @note 根据特定的UserID，查询该用户是否在线。
     */
    virtual bool IsUserOnline(const char *uid) = 0;

    /**  
     * @ingroup CPP_DingRtcEngineChannel
     * @since 3.0
     * @brief 获取状态：当前是否在频道中。
     * @return 
     - - true：表示当前已经加入了频道；
     * - false：表示当前不在频道中。
     * @note 该方法可在加入频道前或加入频道后调用。
     */
    virtual bool IsInCall() = 0;

    // end of ChannelApi
  /**
  * @}
  */

  /**
  * @{ @name PublishApi  推流
  */
    /**
     * @ingroup CPP_DingRtcEnginePub
     * @since 3.0
     * @brief 是否推送本地视频（摄像头）流。
     * @param enable 
     * - 是否开启/关闭本地视频流推送。
     * - true：开启视频流推送。
     * - false：关闭视频流推送。
     * @return
     * - 0：设置成功；
     * - <0：设置失败，返回错误码。
     * @note SDK默认设置不推送视频流，在加入频道前也可以调用此接口修改默认值，并在加入频道成功时生效。
     */
    virtual int PublishLocalVideoStream(bool enable) = 0;

    /**
     * @ingroup CPP_DingRtcEnginePub
     * @since 3.0
     * @brief 是否推送本地音频流。
     * @param enable 
     * - 是否开启/关闭本地音频流推送。
     * - true：开启音频流推送。
     * - false：关闭音频流推送。
     * @return
     * - 0：设置成功；
     * - <0：设置失败，返回错误码。
     * @note SDK默认设置不推送音频流，在加入频道前也可以调用此接口修改默认值，并在加入频道成功时生效。
     */
    virtual int PublishLocalAudioStream(bool enable) = 0;

    /**
     * @ingroup CPP_DingRtcEnginePub
     * @since 3.0
     * @brief 查询是否推送视频流。
     * @return
     * - true：会推送；
     * - false：不会推送。
     */
    virtual bool IsLocalVideoStreamPublished() = 0;

    /**
     * @ingroup CPP_DingRtcEngineScreenShare
     * @since 3.0
     * @brief 是否推送屏幕分享。
     * @return
     * - true：会推送；
     * - false：不会推送。
     * @note SDK默认设置不推送屏幕分享，如果需要推送屏幕共享，可以通过{@link StartScreenShareByDesktopId}， {@link StartScreenShareByWindowId}方法开启屏幕共享推送。
     */
    virtual bool IsScreenSharePublished() = 0;

    /**
     * @ingroup CPP_DingRtcEnginePub
     * @since 3.0
     * @brief 是否推送音频流。
     * @return
     * - true：会推送；
     * - false：不会推送。
     */
    virtual bool IsLocalAudioStreamPublished() = 0;

    /**
     * @ingroup CPP_DingRtcEngineVideo
     * @since 3.0
     * @brief 停止/恢复本地视频数据发送。
     * @param mute
     *          - true表示视频数据发送黑帧。
     *          - false表示恢复正常。
     * @param track 只支持{@link RtcEngineVideoTrackCamera}。
     * @return
     * - 0：成功；
     * - 非0：失败。
     * @note 此接口只是控制指定视频流上是否发送黑帧，采集和数据发送不会停止。
     * @note 如果需要关闭采集请使用 {@link EnableLocalVideo} 接口，如果需要中止视频数据发送请使用 {@link PublishLocalVideoStream} 接口。
     * @note mute状态会保持，除非再次调用MuteLocalVideo更新，或者重新创建engine instance。
     */
    virtual int MuteLocalVideo(bool mute, ding::rtc::RtcEngineVideoTrack track) = 0;

    /**
     * @ingroup CPP_DingRtcEngineAudio
     * @since 3.0
     * @brief 静音/取消静音本地音频数据。
     * @param mute 
     *         - true表示静音本地音频，发送静音帧。
     *         - false表示取消静音。
     * @return 
     * - 0：成功；
     * - 非0：失败。
     * @note mute状态会保持，除非再次调用MuteLocalAudio更新，或者重新创建engine instance。
     */

    virtual int MuteLocalAudio(bool mute) = 0;

    /**
     * @ingroup CPP_DingRtcEngineScreenShare
     * @since 3.0
     * @brief 根据桌面ID进行屏幕分享。
     * @param desktopId 桌面ID（可通过GetScreenShareSourceInfo接口获取）。
     * @param config Reserved. 屏幕分享配置，详见 {@link RtcEngineScreenShareConfig}。
     * @return
     * - 0：成功；
     * - <0：失败。
     * @note config用来设置区域共享，reserved。
     */
    virtual int StartScreenShareByDesktopId(const char* desktopId, const ding::rtc::RtcEngineScreenShareConfig& config) = 0;

    /**
     * @ingroup CPP_DingRtcEngineScreenShare
     * @since 3.0
     * @brief 根据窗口ID进行屏幕分享。
     * @param windowId 窗口ID（可通过GetScreenShareSourceInfo接口获取）。
     * @param config Reserved. 屏幕分享配置，详见 {@link RtcEngineScreenShareConfig}。
     * @return
     * - 0：成功；
     * - <0：失败。
     * @note config用来设置区域共享，reserved。
     */
    virtual int StartScreenShareByWindowId(const char* windowId, const ding::rtc::RtcEngineScreenShareConfig& config) = 0;

#if defined(ALI_OS_IOS)
    /**
     * @brief iOS屏幕分享。
     * @param appGroup 插件和宿主app约定使用相同的appGroup。
     * @param mode 屏幕分享配置，详见 {@link RtcScreenShareMode}。
     * @return
     * - 0：成功；
     * - <0：失败。
     */
   virtual int StartScreenShare(const String& appGroup, ding::rtc::RtcScreenShareMode mode) = 0;
#endif

#if defined(ALI_OS_ANDROID)
    /**
     * @brief android屏幕分享。
     * @param intent 外部创建启动屏幕分享的Activity。
     * @return
     * - 0：成功；
     * - <0：失败。
     */
   virtual int StartScreenShare(void* intent, ding::rtc::RtcScreenShareMode mode) = 0;
#endif

    /**
     * @ingroup CPP_DingRtcEngineScreenShare
     * @since 3.0
     * @brief 停止屏幕分享。
     * @return
     * - 0：成功；
     * - <0：失败。
     */
    virtual int StopScreenShare() = 0;

    /**
     * @ingroup CPP_DingRtcEngineScreenShare
     * @since 3.0
     * @brief 获取屏幕分享源信息。
     * @param sourceType 屏幕分享类型。
     * @return RtcEngineDeviceInfoList 返回屏幕共享源列表，详见 {@link ding::rtc::RtcEngineDeviceInfoList}。
     * @note 请在遍历完屏幕共享源列表后调用其release成员方法，由SDK内部释放相关资源。
     */
    virtual RtcEngineDeviceInfoList* GetScreenShareSourceInfo(ding::rtc::RtcEngineScreenShareType sourceType) = 0;

    /**
     * @ingroup CPP_DingRtcEngineScreenShare
     * @since 3.0
     * @brief 获取当前屏幕共享源Id。
     * @return 当前屏幕共享源Id。
     */
    virtual unsigned int GetCurrentScreenShareSourceId() = 0;

    /**
     * @since 3.0
     * @brief 获取当前屏幕共享源类型。
     * @return RtcEngineScreenShareType 当前屏幕共享源类型，详见 {@link ding::rtc::RtcEngineScreenShareType}。
     */
    virtual RtcEngineScreenShareType GetCurrentScreenShareSourceType() = 0;

    // end of PublishApi
  /**
  * @}
  */

  /**
  * @{ @name SubscribeApi  拉流
  */
    /**
     * @ingroup CPP_DingRtcEngineSub
     * @since 3.0
     * @brief 停止/恢复订阅所有远端音频流，包含后入会成员的订阅。
     * @param sub 
     * - 是否订阅所有远端音频流。
     * - true：订阅所有用户的音频流，后入会成员会自动订阅其音频。
     * - false：停止订阅所有用户的音频流，后入会成员也不会自动订阅其音频。
     * @return
     * - 0：设置成功；
     * - 非0：设置失败。
     * @note
     * - 入会前入会后均可以调用。
     */
    virtual int SubscribeAllRemoteAudioStreams(bool sub) = 0;

    /**
     * @ingroup CPP_DingRtcEngineSub
     * @since 3.0
     * @brief 停止/恢复订阅所有远端视频流，包含后入会成员的订阅。
     * @param sub 
     * - 是否订阅所有远端视频流。
     * - true：订阅所有用户的视频流，后入会成员会自动订阅其视频。
     * - false：停止订阅所有用户的视频流，后入会成员也不会自动订阅其视频。
     * @return
     * - 0：设置成功；
     * - 非0：设置失败。
     * @note
     * - 入会前入会后均可以调用。
     * - 如果设置了 SubscribeAllRemoteVideoStreams(false)，会取消订阅已订阅的远端视频流，同时后面再入会的人不会进行自动订阅；但是依然可以通过 {@link SubscribeRemoteVideoStream} 来主动订阅某个人的远端视频流。
     */
    virtual int SubscribeAllRemoteVideoStreams(bool sub) = 0;

    /**
     * @ingroup CPP_DingRtcEngineSub
     * @since 3.0
     * @brief 停止/恢复订阅远端用户的视频流。
     * @param uid 用户ID，从App server分配的唯一标示符。
     * @param track 视频流类型{@link RtcEngineVideoTrack}。
     * @param sub 
     * - 是否订阅远端用户的视频流。
     * - true：订阅指定用户的视频流。
     * - false：停止订阅指定用户的视频流。
     * @return
     * - 0：设置成功；
     * - 非0：设置失败。
     * @note
     * - 如果设置了 SubscribeAllRemoteVideoStreams(false)，会取消订阅已订阅的远端视频流，同时后面再入会的人不会进行自动订阅；但是依然可以通过 {@link SubscribeRemoteVideoStream} 来主动订阅某个人的远端视频流。
     */
    virtual int SubscribeRemoteVideoStream(const char* uid, ding::rtc::RtcEngineVideoTrack track, bool sub) = 0;

    /**
     * @ingroup CPP_DingRtcEngineSub
     * @since 3.0
     * @brief 设置订阅视频流分辨率规格。
     * @param uid  用户ID，从App server分配的唯一标示符。
     * @param streamType 视频流的分辨率规格。
     * @return
     * - 0：成功；
     * - 非0：失败。
     */
    virtual int SetRemoteVideoStreamType(const char* uid, ding::rtc::RtcEngineVideoStreamType streamType) = 0;

    /**
     * @ingroup CPP_DingRtcEngineSub
     * @since 3.0
     * @brief 设置默认订阅的视频流分辨率规格；如果不设置，则默认订阅最大的分辨率规格。
     * @param streamType 视频流的分辨率规格。
     * @return
     * - 0：成功；
     * - 非0：失败。
     */
    virtual int SetRemoteDefaultVideoStreamType(ding::rtc::RtcEngineVideoStreamType streamType) = 0;

    // end of SubscribeApi
  /**
  * @}
  */

  /**
  * @{ @name CapturerApi  音视频采集
  */
    /**
     * @ingroup CPP_DingRtcEngineVideo
     * @since 3.0
     * @brief 禁用或启用本地视频采集。
     * @param enabled
     *          - true ：启用本地视频采集。
     *          - false ：禁用本地视频采集。
     * @return
     * - 0：成功；
     * - 非0：失败。
     * @note 默认为开启状态。
     */
    virtual int EnableLocalVideo(bool enabled) = 0;

    // end of CapturerApi
  /**
  * @}
  */

  /**
  * @{ @name PreProcessApi  音视频前处理
  */
    /**
     * @brief 开启或关闭视频美颜。
     * @param enable 是否开启。
     * @param options 视频美颜参数。
     * @return
     * - 0：成功；
     * - 非0：失败。
     */
    virtual int EnableBeautyFace(bool enable, ding::rtc::RtcEngineBeautyFaceOptions options) = 0;

    /**
     * @brief 设置视频暗光增强功能。
     * @private
     * @param options 视频暗光增强参数。
     * @return
     * - 0：成功；
     * - 非0：失败。
     */
    virtual int SetVideoEnhance(ding::rtc::RtcEngineVideoEnhanceOptions options) = 0;

    /**
     * @brief 设置视频去噪场景功能。
     * @private
     * @param options 视频去噪参数。
     * @return
     * - 0：成功；
     * - 非0：失败。
     */
    virtual int SetVideoDenoise(ding::rtc::RtcEngineVideoDenoiseOptions options) = 0;

    // end of PreProcessApi
  /**
  * @}
  */

  /**
  * @{ @name EncodeApi  音视频编码
  */
    /**
     * @ingroup CPP_DingRtcEngineVideo
     * @since 3.0
     * @brief 设置相机流视频编码属性。
     * @details 该方法用于设置相机流视频编码属性对应的视频参数，如分辨率、帧率、码率、视频方向等 所有设置的参数都有相应的范围限制，如果设置的参数不在有效范围内，SDK会自动调节。
     * @param config 预定义的编码属性，详见{@link ding::rtc::RtcEngineVideoEncoderConfiguration}。
     * @note
     * - 该方法在入会前和入会后都可以调用，如果每次入会只需要设置一次相机流视频编码属性，建议在入会前调用。
     */
    virtual void SetVideoEncoderConfiguration(const ding::rtc::RtcEngineVideoEncoderConfiguration& config) = 0;

    /**
     * @ingroup CPP_DingRtcEngineVideo
     * @since 3.0
     * @brief 设置屏幕流视频编码属性。
     * @details 该方法用于设置屏幕流视频编码属性对应的视频参数，如分辨率、帧率、码率、视频方向等 所有设置的参数都有相应的范围限制，如果设置的参数不在有效范围内，SDK会自动调节。
     * @param config 预定义的屏幕共享编码属性，详见{@link ding::rtc::RtcEngineScreenShareEncoderConfiguration}。
     * @note
     * - 该方法在入会前和入会后都可以调用，如果每次入会只需要设置一次屏幕流视频编码属性，建议在入会前调用。
     */
    virtual void SetScreenShareEncoderConfiguration(const ding::rtc::RtcEngineScreenShareEncoderConfiguration& config) = 0;
    // end of EncodeApi
  /**
  * @}
  */

  /**
  * @{ @name RenderApi  音视频渲染
  */
    /**
     * @ingroup CPP_DingRtcEngineVideo
     * @since 3.0
     * @brief 设置本地预览显示视图。
     * @param renderConfig 本地视频显示属性。
     * @param track  只支持 {@link RtcEngineVideoTrackCamera} 和 {@link RtcEngineVideoTrackScreen}。
     * @return
     * - 0：成功；
     * - 非0：失败。
     * @note 支持joinChannel之前和之后切换窗口。如果 {@link RtcEngineVideoCanvas::displayView} 为null，则停止显示。
     *       如果需要重新设置{@link RtcEngineVideoCanvas::renderMode}，请保持renderConfig中其他属性不变，仅修改
     *       {@link RtcEngineVideoCanvas::renderMode}。
     *       如果需要重新设置{@link RtcEngineVideoCanvas::mirrorMode}，请保持renderConfig中其他属性不变，仅修改
     *       {@link RtcEngineVideoCanvas::mirrorMode}。
     */
    virtual int SetLocalViewConfig(ding::rtc::RtcEngineVideoCanvas renderConfig, ding::rtc::RtcEngineVideoTrack track) = 0;

    /**
     * @ingroup CPP_DingRtcEngineVideo
     * @since 3.0
     * @brief 设置远端视频显示视图。
     * @param renderConfig 远端视频显示属性。
     * @param uid user id。
     * @param track 只支持 {@link RtcEngineVideoTrackCamera} 和 {@link RtcEngineVideoTrackScreen}。
     * @return
     * - 0：成功；
     * - 非0：失败。
     * @note 如果 {@link RtcEngineVideoCanvas::displayView}为null，则停止显示。
     *       如果需要重新设置 {@link RtcEngineVideoCanvas::renderMode}，请保持renderConfig中其他属性不变，仅修改
     *        {@link RtcEngineVideoCanvas::renderMode}。
     *       如果需要重新设置 {@link RtcEngineVideoCanvas::mirrorMode}，请保持renderConfig中其他属性不变，仅修改
     *        {@link RtcEngineVideoCanvas::mirrorMode}。
     */
    virtual int SetRemoteViewConfig(ding::rtc::RtcEngineVideoCanvas renderConfig, const char *uid, ding::rtc::RtcEngineVideoTrack track) = 0;

    /// @private
    virtual int AddLocalVideoSink(ding::rtc::RtcEngineVideoStreamSink *sink, ding::rtc::RtcEngineVideoTrack track) = 0;
    /// @private
    virtual int RemoveLocalVideoSink(ding::rtc::RtcEngineVideoStreamSink *sink, ding::rtc::RtcEngineVideoTrack track) = 0;
    /// @private
    virtual int AddRemoteVideoSink(ding::rtc::RtcEngineVideoStreamSink *sink, const char *uid, ding::rtc::RtcEngineVideoTrack track) = 0;
    /// @private
    virtual int RemoveRemoteVideoSink(ding::rtc::RtcEngineVideoStreamSink *sink, const char *uid, ding::rtc::RtcEngineVideoTrack track) = 0;

    /**
     * @ingroup CPP_DingRtcEngineVideo
     * @since 3.0
     * @brief 更新渲染配置。
     * @param renderConfig config包含了窗口以及渲染方式。
     * @return
     * - 0：成功；
     * - 非0：失败。
     */
    virtual int UpdateViewConfig(ding::rtc::RtcEngineVideoCanvas renderConfig) = 0;

    // end of RenderApi
  /**
  * @}
  */

  /**
  * @{ @name VideoPreviewApi  视频本地预览
  */
    /**
     * @ingroup CPP_DingRtcEnginePreview
     * @since 3.0
     * @brief 开启本地视频预览。
     * @return
     * - 0：成功；
     * - 非0：失败。
     * @note 如果没有调用 {@link SetLocalViewConfig} 设置显示的view，则无法预览。
     *       可以在{@link JoinChannel}之前就开启预览会自动打开摄像头。
     *       调用LeaveChannel不会关闭本地预览，会一直保留，直到调用StopPreview结束。
     */
    virtual int StartPreview() = 0;

    /**
     * @ingroup CPP_DingRtcEnginePreview
     * @since 3.0
     * @brief 关闭本地视频预览。
     * @return
     * - 0：成功；
     * - 非0：失败。
     * @note {@link LeaveChannel} 会自动停止本地预览；如果没有在推相机流，则会自动关闭摄像头。
     */
    virtual int StopPreview() = 0;

    // end of VideoPreviewApi
  /**
  * @}
  */

  /**
  * @{ @name MediaObserverApi  媒体数据回调和注入
  */
    /**
     * @ingroup CPP_DingRtcEngineVideo
     * @since 3.0
     * @brief 注册视频帧观察类对象。
     * @param observer 视频观察类对象指针。
     * @return
     * - 0：成功；
     * - 非0：失败。
     */
    virtual int RegisterVideoFrameObserver(ding::rtc::RtcEngineVideoFrameObserver* observer) = 0;

    /**
     * @ingroup CPP_DingRtcEngineAudio
     * @since 3.0
    * @brief 设置音频profile。
    * @param audioProfile 音频采集/编码模式参数， 详见{@link RtcEngineAudioProfile}。
    * @param audioScene 音频场景模式参数，详见{@link RtcEngineAudioScenario}。
    * @return 
    * - 0：成功；
    * - 其他：失败。
    * @note 只支持 {@link JoinChannel} 前设置有效。
    */
    virtual int SetAudioProfile(int audioProfile, int audioScene) = 0;

    /**
     * @ingroup CPP_DingRtcEngineAudio
     * @since 3.0
     * @brief 设置音量回调频率和平滑系数。
     * @param interval 时间间隔，单位毫秒，最小值不得小于100ms, 建议设置300-500ms, <= 0表示不启用音量提示和说话人提示功能。
     * @param smooth 平滑系数，数值越大平滑程度越高，反之越低，实时性越好，建议设置3，范围[0, 9]。
     * @param reportVad 说话人检测开关。
     * - 1：开启。
     * - 0：关闭。
     * @return
     * - 0：成功；
     * - <0：失败。
     */
    virtual int EnableAudioVolumeIndication(int interval, int smooth, int reportVad) = 0;

    /**
     * @ingroup CPP_DingRtcEngineAudio
     * @since 3.0
     * @brief 注册音频观察类对象。
     * @private
     * @param observer 音频观察类对象指针。
     * @return
     * - 0：成功；
     * - 非0：失败。
     */
    virtual int RegisterAudioFrameObserver(ding::rtc::RtcEngineAudioFrameObserver* observer) = 0;

    /**
     * @brief 订阅音频数据。
     * @private
     * @param enabled 是否允许数据回调。
     * @param audioSource 音频裸数据源类型，详见 {@link RtcEngineAudioSource}。
     * @return 0：sucess。
     */
    virtual int EnableAudioFrameObserver(bool enabled, RtcEngineAudioSource audioSource) = 0;

    /* 通过渲染检查引擎是否工作正常
     */
    /// @private
    virtual void EnableDetailedStreamObserver(bool enable) = 0;
    /// @private
    virtual bool GetRemoteStreamRendererDetail(const char *uid,
        RtcEngineVideoTrack track,
        VideoRenderDetailInfo &info) = 0;
    /// @private
    virtual bool GetLocalVideoRendererDetail(ding::rtc::VideoRenderDetailInfo &info) = 0;

    /**
     * @ingroup CPP_DingRtcEngineAudio
     * @since 3.0
      * @brief 设置外部音频源参数。
      * @param enable 
      *          - true：开启。
      *          - false：关闭。
      * @param sampleRate 采样率（Hz），取值：8000，16000，32000，44100，48000。
      * @param channels 声道数，取值：1：单声道；2：双声道。
      * @return
      * - >=0：成功；
      * - <0：失败。
      */
      virtual int SetExternalAudioSource(bool enable, int sampleRate, int channels) = 0;

      /**
       * @ingroup CPP_DingRtcEngineAudio
       * @since 3.0
       * @brief 推送外部音频采集源。
       * @param frame 外部音频帧，详见 {@link RtcEngineAudioFrame}。
       * @return
       * - 0：成功；
       * - <0：失败。
       */
      virtual int PushExternalAudioFrame(ding::rtc::RtcEngineAudioFrame* frame) = 0;

      /**
       * @ingroup CPP_DingRtcEngineVideo
       * @since 3.0
       * @brief 设置外部视频输入源。
       * @param enable 是否开启外部视频作为输入。
       * - true 开启。
       * - false 关闭。
       * @param track 视频流类型。
       * @return
       * - 0：成功；
       * - 非0：失败。
       */
      virtual int SetExternalVideoSource(bool enable, ding::rtc::RtcEngineVideoTrack track) = 0;

      /**
       * @ingroup CPP_DingRtcEngineVideo
       * @since 3.0
       * @brief 推送外部视频帧。
       * @param frame 外部视频帧，详见 {@link RtcEngineVideoFrame}。
       * @return
       * - 0：成功；
       * - 非0：失败。
       * @param track 视频流类型{@link RtcEngineVideoTrack}。
       */
      virtual int PushExternalVideoFrame(RtcEngineVideoFrame *frame, ding::rtc::RtcEngineVideoTrack track) = 0;
    // end of MediaObserverApi
  /**
  * @}
  */
    
    /**
     * @since 3.0
     * @brief 开始录制。
     * @details SDK提供了屏录录制的功能，可以录制本地或远端的音频流、视频流、相机流。
     * @param filePath 文件路径。
     * @param name 文件名称。
     * @return
     * - true ：成功；
     * - false ：失败。
     */
    virtual bool StartRecord(ding::rtc::RtcEngineRecordOptions recordOption,
                              const char * filePath,
                              const char * name) = 0;
    
    /**
     * @since 3.0
     * @brief 更新录制内容信息。
     * @details SDK提供了更新录制内容信息的功能，用于实时更新录制模板内容设置。
     * @param layout 录制视频内容及布局， 详细定义见 {@link AliEngineRecordVideoLayout}。
     * @return
     * - true ：成功；
     * - false ：失败。
     * @note 此接口应该在录制过程中调用。
     */
    virtual bool UpdateRecordLayout(ding::rtc::RtcEngineRecordVideoLayout& layout) = 0;
    
    /**
     * @since 3.0
     * @brief 暂停录制。
     * @details SDK提供了暂停录制的功能。
     * @return
     * - true ：成功；
     * - false ：失败。
     */
    virtual bool PauseRecord()  = 0;
    
    /**
     * @since 3.0
     * @brief 重新开始录制。
     * @details SDK提供了重新开始录制的功能。
     * @return
     * - true ：成功；
     * - false ：失败。
     */
    virtual bool ResumeRecord()  = 0;
    
    /**
     * @since 3.0
     * @brief 停止录制。
     * @details SDK提供了停止录制的功能。
     */
    virtual void StopRecord() = 0;
    
    /**
     * @brief 对指定用户的视频进行截图，生成一张 JPG 格式的图片，并保存至指定的路径。
     * @param userId 用户ID， userId 为""代表本地用户。
     * @param trackType 视频流类型，只支持{@link RtcEngineVideoTrackCamera}和{@link RtcEngineVideoTrackScreen}。
     * @param filePath 截图的本地保存路径，需精确到文件名及格式，请确保目录存在且可写。如 xx/yy/zz.jpg 。
     * @return
     * - 0：仅代表调用该接口成功，是否获取到截图画面，详见{@link RtcEngineEventListener::OnSnapshotComplete}；
     * - < 0：失败。
     * @note 视频截图功能为异步接口, 通过监听{@link RtcEngineEventListener::OnSnapshotComplete}回调获取当前截图画面。
     */
    virtual int SnapshotVideo(const char *userId, const RtcEngineVideoTrack &trackType, const char* filePath) = 0;
};

#ifdef __cplusplus
extern "C" {
#endif
    int RtcARGBToNV12(const unsigned char* src_argb, int src_stride_argb, unsigned char* dst_y, int dst_stride_y, unsigned char* dst_uv, int dst_stride_uv, int width, int height);

    int RtcI420ToNV21(const unsigned char* src_y, int src_stride_y, const unsigned char* src_u, int src_stride_u, const unsigned char* src_v, int src_stride_v, unsigned char* dst_y, int dst_stride_y, unsigned char* dst_vu, int dst_stride_vu, int width, int height);

    int RtcI420Rotate(const unsigned char* src_y, int src_stride_y, const unsigned char* src_u, int src_stride_u, const unsigned char* src_v, int src_stride_v, unsigned char* dst_y, int dst_stride_y, unsigned char* dst_u, int dst_stride_u, unsigned char* dst_v, int dst_stride_v, int width, int height, int mode);

    int RtcI420Copy(const unsigned char* src_y, int src_stride_y, const unsigned char* src_u, int src_stride_u, const unsigned char* src_v, int src_stride_v, unsigned char* dst_y, int dst_stride_y, unsigned char* dst_u, int dst_stride_u, unsigned char* dst_v, int dst_stride_v, int width, int height);
#ifdef __cplusplus
}
#endif
//! @}

} // namespace rtc
} // namespace ding

#endif /* __DING_RTC_ENGINE_INTERFACE_H__ */
