#if !defined(__DING_RTC_ENGINE_TYPES_H__)
#define __DING_RTC_ENGINE_TYPES_H__

#include "engine_utils.h"
#include <stdint.h>

namespace ding {
namespace rtc {

//! \addtogroup Basic_Datatypes  数据类型参考
//! @{

/**
 * @ingroup CPP_DingRtcDef
 * @since 3.0
 * @brief 用户离线原因。
 */
typedef enum {
    /** 用户主动离开。 */
    RtcEngineUserOfflineQuit = 0,
    /** 因过长时间收不到对方数据包，超时掉线。 */
    RtcEngineUserOfflineDropped = 1,
} RtcEngineUserOfflineReason;

/**
 * @ingroup CPP_DingRtcDef
 * @since 3.0
 * @brief OnBye类型。
 */
typedef enum {
    /** 当前user被踢出channel。 */
    RtcEngineOnByeBeKickedOut          = 1,
    /** channel已结束，需要离开会议。 */
    RtcEngineOnByeChannelTerminated    = 2,
    /** 相同userID在其他设备 JoinChannel，当前设备被下线。 */
    RtcEngineOnByeUserReplaced         = 3,
    /** 信令服务失败，自动重连失败，需要用户手动重试。 */
    RtcEngineOnByeNeedRelogin          = 4,
} RtcEngineOnByeType;

/**
 * @ingroup CPP_DingRtcDef
 * @since 3.0
 * @brief 用户鉴权信息。
 */
struct RtcEngineAuthInfo {
    /*! 频道ID。 */ 
    String channelId;
    /*! 用户ID。 */
    String userId;
    /*! 应用ID。 */
    String appId;
	/*! 令牌。 */
    String token;
	/*! GSLB地址。 */
    String gslbServer;
};

/**
 * @ingroup CPP_DingRtcDef
 * @since 3.0
 * @brief 网络连接状态。
 */
typedef enum {
    /** 初始化完成。 */
    RtcEngineConnectionInit = 0,
    /** 网络连接断开。 */
    RtcEngineConnectionDisconnected,
    /** 建立网络连接中。 */
    RtcEngineConnectionConnecting,
    /** 网络已连接。 */
    RtcEngineConnectionConnected,
    /** 重新建立网络连接中。 */
    RtcEngineConnectionReconnecting,
    /** 网络连接失败。 */
    RtcEngineConnectionFailed,
} RtcEngineConnectionStatus;

/**
 * @ingroup CPP_DingRtcDef
 * @since 3.0
 * @brief 网络连接状态变更原因。
 */
typedef enum {
    /** 未知原因。 */
    RtcEngineConnectionChangedDummyReason = 0,
    /** 心跳超时。 */
    RtcEngineConnectionSignalingHeartbeatTimeout,
    /** 心跳恢复。 */
    RtcEngineConnectionSignalingHeartbeatAlive,
    /** 加入频道失败。 */
    RtcEngineConnectionSignalingJoinChannelFailure,
    /** 加入频道成功。 */
    RtcEngineConnectionSignalingJoinChannelSuccess,
    /** 离开频道。 */
    RtcEngineConnectionSignalingLeaveChannel,
    /** 信令建立连接。 */
    RtcEngineConnectionSignalingConnecting,
    /** 网络连接中断。 */
    RtcEngineConnectionChangedNetworkInterrupted,
} RtcEngineConnectionStatusChangeReason;

/**
 * @ingroup CPP_DingRtcDef
 * @since 3.0
 * @brief 网络质量。
 */
typedef enum {
    /** 网络质量好。 */
    RtcEngineNetworkQualityGood          = 0,
    /** 网络质量差。 */
    RtcEngineNetworkQualityPoor          = 1,
    /** 网络中断。 */
    RtcEngineNetworkQualityDisconnect    = 2,
    /** 未知。 */
    RtcEngineNetworkQualityUnknown       = 3,
} RtcEngineNetworkQuality;

/**
 * @ingroup CPP_DingRtcDef
 * @since 3.0
 * @brief 日志级别。
 */
typedef enum {
    /** 只输出>=RtcEngineLogLevelInfo 级别的日志。 */
    RtcEngineLogLevelInfo = 3,
    /** 只输出>=RtcEngineLogLevelWarn 级别的日志。 */
    RtcEngineLogLevelWarn = 4,
    /** 只输出>=RtcEngineLogLevelError 级别的日志。 */
    RtcEngineLogLevelError = 5,
    /** 不输出日志。 */
    RtcEngineLogLevelNone = 6,
} RtcEngineLogLevel;

/**
 * @ingroup CPP_DingRtcDef
 * @since 3.0
 * @brief SDK错误码。
 */
typedef enum {
    /****************************************************
     * 入会错误码
     ****************************************************/
    /** AppId不存在，请在控制台创建应用。 */
    RtcEngineErrorJoinBadAppId = 0x02010201,
    /** AppId已失效，请在控制台重新启用应用。 */
    RtcEngineErrorJoinInvaildAppId = 0x02010202,
    /**
     * 频道不存在，请尝试重新加入频道，如会议发起者入会后又很快离会，第二个人随后入会，发现会议已经不存在了。
     */
    RtcEngineErrorJoinBadChannel = 0x02010204,
    /** 频道已经失效（目前频道有效期为48小时），请重新生成频道鉴权令牌（Token）。 */
    RtcEngineErrorJoinInvaildChannel = 0x02010203,
    /** Token无效。 请重新生成频道鉴权令牌（Token）。 */
    RtcEngineErrorJoinBadToken = 0x02010205,
    /** 加入频道超时，请检查网络连接是否正常。 */
    RtcEngineErrorJoinTimeout = 0x01020204,
    /** 参数错误，{@link RtcEngineAuthInfo}中的字段为空或者timestamp<=0，请检查字段是否完整。 */
    RtcEngineErrorJoinBadParam = 0x01030101,
    /** 加入频道失败。 */
    RtcEngineErrorJoinChannelFailed = 0x01030202,

    /****************************************************
     * 日志模块错误码
     ****************************************************/
    /** 日志模块根路径错误，请检查路径是否存在或路径是否具有写入权限。 */
    RtcEngineErrorLogRootPathError = 0x01030106,
    
    /****************************************************
     * 音频设备错误码
     ****************************************************/
    /**
     * 采集设备初始化失败、采集设备媒体服务异常、音频设备被占用。
     * - Mac端和Windows端查看系统设置中音频设备是否有能量条。
     * - iOS端和Android端确认是否被其他应用占用。
     * - 如果都正常，请重启设备。
     */
    RtcEngineErrorMicOpenFail = 0x01040404,
    /**
     * 播放设备初始化失败、播放设备媒体服务异常、音频设备被占用。
     * - Mac端和Windows端查看系统设置中音频设备是否有能量条。
     * - iOS端和Android端确认是否被其他应用占用。
     * - 如果都正常，请重启设备。
     */
    RtcEngineErrorSpeakerOpenFail = 0x01040405,
    /** 系统性能不足、采集设备媒体服务异常、音频设备被占用。建议重启应用或重启设备。 */
    RtcEngineErrorMicInterrupt = 0x01040406,
    /** 系统性能不足，播放设备媒体服务异常，音频设备被占用。建议重启应用或重启设备。 */
    RtcEngineErrorSpeakerInterrupt = 0x01040407,
    /** 麦克风设备未授权，建议去系统设置中，打开麦克风授权，并且重新加入频道。 */
    RtcEngineErrorMicAuthFail = 0x01040408,
    /** 无可用的音频采集设备，检查是否接入了可用的麦克风，并检查系统设置中，该设备是否被禁用。 */
    RtcEngineErrorMicNotAvailable = 0x01040409,
    /** 无可用的音频播放设备，检查是否接入了可用的扬声器或耳机，并检查系统设置中，该设备是否被禁用。 */
    RtcEngineErrorSpeakerNotAvailable = 0x01040410,
    /** 指定使用的音频设备未找到。 */
    RtcEngineErrorSpecifiedAudioDeviceNotFound = 0x01040411,

    /****************************************************
     * 视频设备错误码
     ****************************************************/
    /** 采集设备初始化失败，检查系统相机是否可用，移动端App是否占用了相机。 */
    RtcEngineErrorCameraOpenFail = 0x01040104,
    /** 采集过程中出现异常，Mac、Winodows电脑相机采集灯熄灭，设备被强制中断。建议重新加入频道或者重启App。 */
    RtcEngineErrorCameraInterrupt = 0x01040106,
    /** 指定使用的摄像头设备未找到。 */
    RtcEngineErrorSpecifiedCameraDeviceNotFound = 0x01040107,
    /** 屏幕共享打开失败。 */
    RtcEngineErrorScreenShareOpenFail = 0x01040108,
    /** 渲染设备初始化失败，检查Windows端显卡驱动是否需要升级，移动端手机型号或者Android平板盒子等外设是否支持OpenGL。 */
    RtcEngineErrorVideoDisplayOpenFail = 0x01040201,
    /** 渲染过程中出现异常，系统异常错误导致渲染引擎报错，重新加入频道或者重启App。 */
    RtcEngineErrorVideoDisplayInterrupt = 0x01040202,

    /****************************************************
     * 网络错误码
     ****************************************************/
    /** 媒体通道建立失败，检查网络连接是否正常。 */
    RtcEngineErrorIceConnectionConnectFail = 0x01050201,
    /** 媒体通道重连失败，检查网络连接是否正常。 */
    RtcEngineErrorIceConnectionReconnectFail = 0x01050202,
    /** 信令心跳超时，检查网络连接是否正常。 */
    RtcEngineErrorConnectionHeartbeatTimeout = 0x0102020C,
    /** 信令链接失败，检查信令网络连接是否正常。 */
    RtcEngineErrorConnectionConnectFail = 0x01050301,

    /****************************************************
     * 推流相关错误码
     ****************************************************/
    /** 推流无效。 */
    RtcEngineErrorPublishInvaild = 0x01030305,
    /** 未进入频道推流失败。 */
    RtcEngineErrorPublishNotJoinChannel = 0x01010406,
    /** 推送音频流失败。 */
    RtcEngineErrorPublishAudioStreamFailed = 0x01010450,
    /** 推送视频流失败。 */
    RtcEngineErrorPublishVideoStreamFailed = 0x01010451,
    /** 推送屏幕共享失败。 */
    RtcEngineErrorPublishScreenShareFailed = 0x01010453,
    /** 屏幕共享配置错误。 */
    RtcEngineErrorPublishScreenShareConfigError = 0x01010454,

    /****************************************************
     * 订阅相关错误码
     ****************************************************/
    /** 订阅无效 */
    RtcEngineErrorSubscribeInvaild = 0x01030404,
    /** 未进入频道订阅错误。*/
    RtcEngineErrorSubscribeNotJoinChannel = 0x01010550,
    /** 订阅音频流失败。 */
    RtcEngineErrorSubscribeAudioStreamFailed = 0x01010551,
    /** 订阅视频流失败。 */
    RtcEngineErrorSubscribeVideoStreamFailed = 0x01010552,
    /** 订阅屏幕共享失败。 */
    RtcEngineErrorSubscribeScreenShareFailed = 0x01010554,

    /****************************************************
     * 其他错误码
     ****************************************************/
    /** sdk状态错误，建议销毁SDK重新创建实例。 */
    RtcEngineErrorInvaildState = 0x01030204,
    /** 参数不匹配。 */
    RtcEngineErrorInvaildArgument = 0x01030103,
    /** Session已经被移除，建议销毁实例之后重新创建实例并加入频道，已废弃。 */
    RtcEngineErrorSessionRemoved = 0x02010105,
    /** buffer队列饱和，用于外部输送裸数据功能，建议降低外部输入频次，设置等待20ms，并且单次不超过40ms数据。 */
    RtcEngineErrorAudioBufferFull = 0x01070101,
    /** buffer队列饱和，用于外部输送裸数据功能，建议降低外部输入频次，设置等待视频帧间隔后再输入。 */
    RtcEngineErrorVideoBufferFull = 0x01070102,
    /** 音频流错误。 */
    RtcEngineErrorAudioStreamError = 0x01080101,
    /** 音频文件打开失败。 */
    RtcEngineErrorAudioFileOpenFail = 0x01080102,
    /** 音频格式不支持。 */
    RtcEngineErrorAudioFormatNotSupported = 0x01080103,
    /** 互动模式下设置角色错误。 */
    RtcEngineErrorUpdateRoleChannel = 0x01020201,
    /** SDK内部错误。 */
    RtcEngineErrorInner = -1,
    
} RtcEngineErrorCode;

/**
 * @ingroup CPP_DingRtcDef
 * @since 3.0
 * @brief 视图显示模式。
 */
typedef enum {
    /** 自动模式。 */
    RtcEngineRenderModeAuto     = 0,
    /** 拉伸平铺模式 ，如果外部输入的视频宽高比和推流设置的宽高比不一致时，将输入视频拉伸到推流设置的比例，画面会变形。 */
    RtcEngineRenderModeStretch  = 1,
    /** 填充黑边模式，如果外部输入的视频宽高比和推流设置的宽高比不一致时，将输入视频上下或者左右填充黑边。 */
    RtcEngineRenderModeFill     = 2,
    /** 裁剪模式，如果外部输入的视频宽高比和推流设置的宽高比不一致时，将输入视频宽或者高进行裁剪，画面内容会丢失。 */
    RtcEngineRenderModeCrop     = 3,
    /** 沿用之前的显示模式。 */
    RtcEngineRenderModeNoChange = 99,
} RtcEngineRenderMode;

/**
 * @ingroup CPP_DingRtcDef
 * @since 3.0
 * @brief 视频窗口镜像模式。
 */
typedef enum {
    /** 只有前置摄像头预览镜像，其余不镜像。 */
    RtcEngineRenderMirrorModeOnlyFrontMirror  = 0,
    /** 镜像开启。 */
    RtcEngineRenderMirrorModeAllMirror        = 1,
    /** 镜像关闭。 */
    RtcEngineRenderMirrorModeAllNoMirror      = 2,
    /** 占位符，无意义。 */
    RtcEngineRenderMirrorModeNoChange         = 99,
} RtcEngineRenderMirrorMode;

/**
 * @ingroup CPP_DingRtcDef
 * @since 3.0
 * @brief 视频宽高比。
 */
typedef enum  {
    /** 16 : 9。 */
    RtcEngineVideoScale_16_9 = 0,
    /** 4 : 3。 */
    RtcEngineVideoScale_4_3,
}RtcEngineVideoScale;

/**
 * @ingroup CPP_DingRtcDef
 * @since 3.0
 * @brief 视频旋转角度。
 */
typedef enum {
    /** 沿用之前的旋转角度。 */
    RtcEngineRotationModeNoChange = -1,
    /** 旋转0度。 */
    RtcEngineRotationMode_0 = 0,
    /** 旋转90度。 */
    RtcEngineRotationMode_90 = 90,
    /** 旋转180度。 */
    RtcEngineRotationMode_180 = 180,
    /** 旋转270度。 */
    RtcEngineRotationMode_270 = 270,
} RtcEngineRotationMode;

/**
 * @ingroup CPP_DingRtcDef
 * @since 3.0
 * @brief 视频显示窗口设置。
 */
struct RtcEngineVideoCanvas {
    /** 显示视图。 */
    void *displayView   = nullptr;
    /** 背景颜色，格式为RGB的Hex。 */
    int backgroundColor = 0;
    /** 渲染模式，默认值为{@link RtcEngineRenderModeAuto}。 */
    RtcEngineRenderMode renderMode = RtcEngineRenderMode::RtcEngineRenderModeAuto;
    /** 镜像模式，默认值为{@link RtcEngineRenderMirrorModeOnlyFrontMirror}。 */
    RtcEngineRenderMirrorMode mirrorMode = RtcEngineRenderMirrorModeOnlyFrontMirror;
    /** 视频宽高比，默认值为 {@link RtcEngineVideoScale_16_9}。 */
    RtcEngineVideoScale scaleMode = RtcEngineVideoScale_16_9;
    /** 旋转角度，默认值为 {@link RtcEngineRotationMode_0}。 */
    RtcEngineRotationMode rotation = RtcEngineRotationMode_0;
    /** 支持第三方OpenGL ES纹理显示，纹理ID。 */
    long long renderId = 0;
    /** 支持第三方OpenGL ES纹理显示，纹理宽。 */
    int width = 0;
    /** 支持第三方OpenGL ES纹理显示，纹理高。 */
    int height = 0;
    /** 该参数只在iOS和Mac平台生效，
     * - true：使用OpenGL ES 渲染。
     * - false：使用Metal渲染。
     */
    bool enableGL = true;
    /**
     * 是否移除视图。
     */
    bool toBeRemoved = false;
};

/**
 * @ingroup CPP_DingRtcDef
 * @since 3.0
 * @brief 音频track的类型。
 */
typedef enum {
    /** 无麦克风。 */
    RtcEngineAudioTrackNo = 0,
    /** 麦克风。 */
    RtcEngineAudioTrackMic = 1,
    /** 占位符。 */
    RtcEngineAudioTrackEnd = 0xffffffff
} RtcEngineAudioTrack;

/**
 * @ingroup CPP_DingRtcDef
 * @since 3.0
 * @brief 音频帧类型。
 */
typedef enum {
    RtcEngineAudioFramePcm16 = 0,  // PCM 16bit little endian
} RtcEngineAudioFrameType;

/**
 * @ingroup CPP_DingRtcDef
 * @since 3.0
 * @brief 音频帧数据结构。
 */
typedef struct {
    /** 音频帧类型。 */
    RtcEngineAudioFrameType type;
    /** 每个音频采样字节数, 通常是16bit（即2字节）。 */
    int bytesPerSample;
    /** 采样率。 */
    int samplesPerSec;
    /** 声道数：1：单声道，2：双声道。 */
    int channels;
    /** 采样点数（每个声道）。 */
    int samples;
    /** 音频帧数据缓冲区，该缓冲区大小为：`buffer_size` = `samples` × `channels` × `bytesPerSample`。 */
    void* buffer;
    /** 音频时间戳。 */
    long long timestamp;
} RtcEngineAudioFrame;

/**
 * @ingroup CPP_DingRtcDef
 * @since 3.0
 * @brief 视频流类型。
 */
typedef enum {
    /** 无视频流。 */
    RtcEngineVideoTrackNo     = 0,
    /** 相机流。 */
    RtcEngineVideoTrackCamera = 1,
    /** 屏幕共享流。 */
    RtcEngineVideoTrackScreen = 2,
    /** 相机流和屏幕共享流。 */
    RtcEngineVideoTrackBoth   = 3,
} RtcEngineVideoTrack;

/**
 * @ingroup CPP_DingRtcDef
 * @since 3.0
 * @brief 视频码流类型。
 */
typedef enum {
    /** 无，在OnSubscribeStreamTypeChanged回调表示当前未订阅。 */
    RtcEngineVideoStreamTypeNone = 0,
    /** 超高清流（设备性能、网络状态、推流分辨率符合要求情况下满足），对应1080P收费档。 */
    RtcEngineVideoStreamTypeFHD = 1,
    /** 高清流（设备性能、网络状态、推流分辨率符合要求情况下满足），对应720P收费档。 */
    RtcEngineVideoStreamTypeHD = 2,
    /** 标清（设备性能、网络状态、推流分辨率符合要求情况下满足），对应480P收费档。 */
    RtcEngineVideoStreamTypeSD = 3,
    /** 流畅（设备性能、网络状态、推流分辨率符合要求情况下满足），对应480P收费档。 */
    RtcEngineVideoStreamTypeLD = 4,
} RtcEngineVideoStreamType;

/**
 * @ingroup CPP_DingRtcDef
 * @since 3.0
 * @brief 订阅状态。
 */
typedef enum {
    /** 初始状态。 */
    RtcEngineStatsSubscribeIdle = 0,
    /** 未订阅。 */
    RtcEngineStatsNoSubscribe = 1,  
    /** 订阅中。 */  
    RtcEngineStatsSubscribing = 2,  
    /** 已订阅。 */  
    RtcEngineStatsSubscribed = 3,     
} RtcEngineSubscribeState;

/**
 * @ingroup CPP_DingRtcDef
 * @since 3.0
 * @brief 采集偏好。
 */
typedef enum {
    /** 自动调整采集参数（默认）。 */
    RtcEngineCaptureOutputPreferenceAuto = 0,
    /** 根据设置的publsh profile 分辨率和帧率，选择最接近的摄像头输出参数。 */
    RtcEngineCaptureOutputPreferencePerformance = 1,
    /** 选择较高的摄像头输出参数。 */
    RtcEngineCaptureOutputPreferencePreview = 2,
}RtcEngineCaptureOutputPreference;

/**
 * @ingroup CPP_DingRtcDef
 * @since 3.0
 * @brief 摄像头方向。
 */
typedef enum {
    /** 无效，不可用。 */
    RtcEngineCameraDirectionInvalid = -1,
    /** 后置摄像头。 */
    RtcEngineCameraDirectionBack = 0,
    /** 前置摄像头。 */
    RtcEngineCameraDirectionFront = 1,
 }RtcEngineCameraDirection;

/**
 * @ingroup CPP_DingRtcDef
 * @since 3.0
 * @brief camera 采集偏好。
 */
typedef struct {
    /** 采集偏好。 */
    RtcEngineCaptureOutputPreference preference;        /** Camera capturer preference settings. See: #RtcEngineCaptureOutputPreference. */
#if defined(ALI_OS_ANDROID) || defined(ALI_OS_IOS)
    RtcEngineCameraDirection cameraDirection;           /** Camera direction settings (for Android/iOS only). See: #RtcEngineCameraDirection. */
#endif
} RtcEngineCameraCapturerConfiguration;

/**
 * @ingroup CPP_DingRtcDef
 * @since 3.0
 * @brief 视频输出方向。
 */
enum RtcEngineVideoEncoderOrientationMode {
    /** 0：自适应，推流方向和采集方向一致。
     */
    RtcEngineVideoEncoderOrientationModeAdaptive = 0,
    /** 1：竖屏模式：该模式下SDK推竖屏流，始终以设置的分辨率宽和高中较小的值作为输出视频的宽，较大值作为输出视频的高。
     */
    RtcEngineVideoEncoderOrientationModeFixedLandscape = 1,
    /** 2：横屏模式：该模式下SDK推横屏流，始终以设置的分辨率宽和高中较大的值作为输出视频的宽，较小值作为输出视频的高。
     */
    RtcEngineVideoEncoderOrientationModeFixedPortrait = 2,
};

/**
 * @ingroup CPP_DingRtcDef
 * @since 3.0
 * @brief 视频镜像模式。
 */
enum RtcEngineVideoMirrorMode
{
    /** 0：关闭镜像。
    */
    RtcEngineVideoMirrorModeDisabled = 0,//disable mirror
    /** 1：开启镜像。
     */
    RtcEngineVideoMirrorModeEnable = 1,//enabled mirror
};

/**
 * @ingroup CPP_DingRtcDef
 * @since 3.0
 * @brief 视频分辨率。
 */
struct RtcEngineVideoDimensions {
    /** 分辨率宽。
     */
    int width;
    /** 分辨率高。
     */
    int height;
    RtcEngineVideoDimensions()
        : width(640), height(480)
    {}
    RtcEngineVideoDimensions(int w, int h)
        : width(w), height(h)
    {}
};

/**
 * @ingroup CPP_DingRtcDef
 * @since 3.0
 * @brief 视频美颜选项。
 */
struct RtcEngineBeautyFaceOptions {
    /** 资源文件路径。
     */
    const char* resourcePath;
    /** 开启美肤。
     */
    bool enableSkinBuffing;
    /** 磨皮系数。
     */
    float skinBuffingFactor;
    /** 锐化系数。
     */
    float skinSharpenFactor;
    /** 美白系数。
     */
    bool enableSkinWhitening;
    /** 美白系数。
     */
    float skinWhitingFactor;

    RtcEngineBeautyFaceOptions()
        : resourcePath(nullptr)
        , enableSkinBuffing(false)
        , skinBuffingFactor(0.6f)
        , skinSharpenFactor(0.8f)
        , enableSkinWhitening(false)
        , skinWhitingFactor(0.8f)
    {}
};

/**
 * @ingroup CPP_DingRtcDef
 * @since 3.0
 * @brief 视频暗光增强选项。
 */
enum RtcEngineVideoEnhanceMode {
    /** 视频暗光增强关。
     */
    RtcEngineVideoEnhanceOff = 0,
    /** 视频暗光增强自动模式。
     */
    RtcEngineVideoEnhanceAuto = 1,
};

/**
 * @ingroup CPP_DingRtcDef
 * @since 3.0
 * @brief 视频暗光增强选项。
 */
struct RtcEngineVideoEnhanceOptions {
    /** 视频暗光增强模式。
     */
    RtcEngineVideoEnhanceMode mode = RtcEngineVideoEnhanceOff;
};

/**
 * @ingroup CPP_DingRtcDef
 * @since 3.0
 * @brief 视频去噪模式。
 */
enum RtcEngineVideoDenoiseMode {
    /** 视频去噪关。
     */
    RtcEngineVideoDenoiseOff = 0,
    /** 视频去噪自动模式。
     */
    RtcEngineVideoDenoiseAuto = 1,
};

/**
 * @ingroup CPP_DingRtcDef
 * @since 3.0
 * @brief 视频去噪选项。
 */
struct RtcEngineVideoDenoiseOptions {
    /** 视频去噪模式。
     */
    RtcEngineVideoDenoiseMode mode = RtcEngineVideoDenoiseOff;
};

/**
 * @brief 预定义视频分辨率
 */
const RtcEngineVideoDimensions VD_120x120(120, 120);
const RtcEngineVideoDimensions VD_160x120(160, 120);
const RtcEngineVideoDimensions VD_180x180(180, 180);
const RtcEngineVideoDimensions VD_240x180(240, 180);
const RtcEngineVideoDimensions VD_320x180(320, 180);
const RtcEngineVideoDimensions VD_240x240(240, 240);
const RtcEngineVideoDimensions VD_320x240(320, 240);
const RtcEngineVideoDimensions VD_424x240(424, 240);
const RtcEngineVideoDimensions VD_360x360(360, 360);
const RtcEngineVideoDimensions VD_480x360(480, 360);
const RtcEngineVideoDimensions VD_640x360(640, 360);
const RtcEngineVideoDimensions VD_480x480(480, 480);
const RtcEngineVideoDimensions VD_640x480(640, 480);
const RtcEngineVideoDimensions VD_840x480(840, 480);
const RtcEngineVideoDimensions VD_960x540(960, 540);
const RtcEngineVideoDimensions VD_960x720(960, 720);
const RtcEngineVideoDimensions VD_1280x720(1280, 720);
const RtcEngineVideoDimensions VD_1920x1080(1920, 1080);

/**
 * @ingroup CPP_DingRtcDef
 * @since 3.0
 * @brief 相机流视频编码属性设置。
 */
struct RtcEngineVideoEncoderConfiguration{
    
    /** 视频分辨率，默认值640x480，可自行设置分辨率，也可以选择上面预定义的分辨率。
     */
    RtcEngineVideoDimensions dimensions;
    
    /** 视频帧率，默认值20, 最大值30。
    */
    int frameRate;
    /** 视频输出方向，默认RtcEngineVideoEncoderOrientationModeAdaptive，详见 {@link ding::rtc::RtcEngineVideoEncoderOrientationMode}。
    */
    RtcEngineVideoEncoderOrientationMode orientationMode;
    
    RtcEngineVideoEncoderConfiguration(
        const RtcEngineVideoDimensions& d, int f,
        RtcEngineVideoEncoderOrientationMode m)
        : dimensions(d), frameRate(f),
          orientationMode(m)
    {}
    RtcEngineVideoEncoderConfiguration(
        int width, int height, int f,
        RtcEngineVideoEncoderOrientationMode m)
        : dimensions(width, height), frameRate(f),
          orientationMode(m)
    {}
    RtcEngineVideoEncoderConfiguration()
        : dimensions(640, 480)
        , frameRate(20)
        , orientationMode(RtcEngineVideoEncoderOrientationModeAdaptive)
    {}
    RtcEngineVideoEncoderConfiguration(const RtcEngineVideoEncoderConfiguration& config)
        : dimensions(config.dimensions.width, config.dimensions.height)
        , frameRate(config.frameRate)
        , orientationMode(config.orientationMode)
    {}
};

/**
 * @ingroup CPP_DingRtcDef
 * @since 3.0
 * @brief 屏幕流编码属性设置。
 */
struct RtcEngineScreenShareEncoderConfiguration{
    /** 视频分辨率，默认值0x0，最大值3840x2160。
     @note
     默认值表示推流分辨率等于屏幕采集的分辨率。
    */
    RtcEngineVideoDimensions dimensions;
    
    /** 视频帧率，默认值5, 最大值30。
    */
    int frameRate;
    
    RtcEngineScreenShareEncoderConfiguration(
        const RtcEngineVideoDimensions& d, int f)
        : dimensions(d), frameRate(f)
    {}
    RtcEngineScreenShareEncoderConfiguration(
        int width, int height, int f)
        : dimensions(width, height), frameRate(f)
    {}
    RtcEngineScreenShareEncoderConfiguration()
        : dimensions(0, 0)
        , frameRate(5)
    {}
    RtcEngineScreenShareEncoderConfiguration(const RtcEngineScreenShareEncoderConfiguration& config)
        : dimensions(config.dimensions.width, config.dimensions.height)
        , frameRate(config.frameRate)
    {}
};

/**
 * @ingroup CPP_DingRtcDef
 * @since 3.0
 * @brief 桌面分享类型。
 */
typedef enum {
    /** 桌面分享。 */
    RtcEngineScreenShareDesktop, 
    /** 窗口分享。 */     
    RtcEngineScreenShareWindow,       
} RtcEngineScreenShareType;

/**
 * @ingroup CPP_DingRtcDef
 * @since 3.0
 * @brief 屏幕分享区域。
 */
struct RtcEngineScreenShareRegion {
    /** x轴。 */  
    float originX = -1.f;
    /** y轴。 */  
    float originY = -1.f;
    /** 宽度。 */  
    float width = 0.f;
    /** 高度。 */  
    float height = 0.f;
};

#if defined(ALI_OS_ANDROID) || defined(ALI_OS_IOS)
/**
 * @brief iOS屏幕共享模式。
 */
typedef enum {
    /** 不共享音视频流。 */
    RtcScreenShareNone       = 0,
    /** 共享视频流。 */
    RtcScreenShareVideo      = 1 << 0,
    /** 共享App音频流。 */
    RtcScreenShareAudioApp   = 1 << 1,
    /** 共享所有音视频流 , 不支持屏幕共享Mic音频流, 请使用SDK自带的Mic音频采集。 */
    RtcScreenShareAll        = 3,
} RtcScreenShareMode;
#endif

/**
 * @ingroup CPP_DingRtcDef
 * @since 3.0
 * @brief 屏幕共享参数配置。
 */
struct RtcEngineScreenShareConfig {
    /** 是否指定区域，默认false。 */
    bool isShareByRegion = false;
    /** 屏幕分享区域。 */
    RtcEngineScreenShareRegion shareRegion;

    void clear()
    {
        shareRegion.originX = -1.f;
        shareRegion.originY = -1.f;
        shareRegion.width = 0.f;
        shareRegion.height = 0.f;
    }
};

/**
 * @ingroup CPP_DingRtcDef
 * @since 3.0
 * @brief 屏幕共享配置信息。
 */
struct RtcEngineScreenShareInfo {
    /** 桌面分享类型。 */
    RtcEngineScreenShareType shareType = RtcEngineScreenShareDesktop;
    /** 共享源ID。 */
    unsigned int shareSourceId = 0;
    /** 屏幕共享参数配置。 */
    RtcEngineScreenShareConfig shareConfig;
};

/**
 * @ingroup CPP_DingRtcDef
 * @since 3.0
 * @brief 推流状态。
 */
typedef enum {
  /** 初始状态。 */
  RtcEngineStatsPublishIdle = 0,
  /** 未推流。 */
  RtcEngineStatsNoPublish = 1,
  /** 推流中。 */
  RtcEngineStatsPublishing = 2,
  /** 已推流。 */
  RtcEngineStatsPublished = 3,
} RtcEnginePublishState;

/**
 * @ingroup CPP_DingRtcDef
 * @since 3.0
 * @brief 视频帧类型。
 */
typedef enum {
  /** 裸数据类型。 */
  RtcEngineVideoFrameRaw = 0,
  /** CVPixelBufferRef 类型。 */
  RtcEngineVideoFramePixelBuffer = 1,
  /** 纹理类型。 */
  RtcEngineVideoFrameTexture = 2
} RtcEngineVideoFrameType;

/**
 * @ingroup CPP_DingRtcDef
 * @since 3.0
 * @brief 视频数据格式。
 */
typedef enum {
  /** I420。 */
  RtcEngineVideoI420,
  /** NV12。 */
  RtcEngineVideoNV12,
  /** NV21。 */
  RtcEngineVideoNV21,
  /** BGRA */
  RtcEngineVideoBGRA,
  /** ARGB。 */
  RtcEngineVideoARGB,
  /** RGBA。 */
  RtcEngineVideoRGBA,
  /** ABGR。 */
  RtcEngineVideoABGR,
  /** 纹理。 */
  RtcEngineVideoTexture,
} RtcEngineVideoPixelFormat;

const int kMaxVideoPlaneCount = 3;

/**
 * @ingroup CPP_DingRtcDef
 * @since 3.0
 * @brief 视频帧数据结构。
 */
typedef struct {
  /** 视频帧类型。 */  
  RtcEngineVideoFrameType frameType;
  /** 视频数据格式。 */
  RtcEngineVideoPixelFormat pixelFormat;
  /** 时间戳。 */
  long long timestamp;
  /** 视频帧的宽。 */
  int width;
  /** 视频帧的高。 */
  int height;
  /** 视频旋转角度。 */
  int rotation;
  /** 视频帧的数量。 */
  int count;
  int offset[kMaxVideoPlaneCount];
  /** 视频帧行间距。 */
  int stride[kMaxVideoPlaneCount];
  /** 视频帧数据。 */
  void *data;
  /** 纹理ID。 */
  int textureId;
} RtcEngineVideoFrame;

/* 
 * @ingroup CPP_DingRtcDef
 * @since 3.0
 * @brief 视频渲染信息。
 */
typedef struct {
    /* render fps */
    int fps_num;
    int fps_den;
    /* size */
    int width; // 0: (no frames yet)
    int height;
    /* rotation */
    int rotation;
    /* smoothness (帧间隔方差) */
    int variance_num; // -1: meaningless
    int variance_den;
    /* 黑帧检测。 */
    int black; // 0: not black frame; 1: black frame; -1: unknown
} VideoRenderDetailInfo;

/**
 * @ingroup CPP_DingRtcDef
 * @since 3.0
 * @brief 会话数据统计信息。
*/
typedef struct {
    /** 通话时长，单位：秒。 */
    unsigned long long duration;
    
    /** 发送字节数。 */
    unsigned long long txBytes;
    /** 音频发送字节数。 */
    unsigned long long txAudioBytes;
    /** 视频发送字节数。 */
    unsigned long long txVideoBytes;
    /** 发送码率。 */
    unsigned short txKBitrate;
    /** 音频发送码率。 */
    unsigned short txAudioKBitrate;
    /** 视频发送码率 */
    unsigned short txVideoKBitrate;
    /** 客户端到服务器的丢包率。 */
    int txPacketLossRate;

    /** 接收字节数。 */
    unsigned long long rxBytes;
    /** 接收数据包。 */
    unsigned long long rxPackets;
    /** 音频接收字节数。 */
    unsigned long long rxAudioBytes;
    /** 视频接收字节数。 */
    unsigned long long rxVideoBytes;
    /** 接收码率。 */
    unsigned short rxKBitrate;
    /** 音频接收码率。 */
    unsigned short rxAudioKBitrate;
    /** 视频接收码率。 */
    unsigned short rxVideoKBitrate;
    /** 客户端到服务器的下行丢包率。 */
    int rxPacketLossRate;
    /** 客户端到服务器的下行丢包数。 */
    unsigned long long rxLostPackets;

    /** 延时时长，单位：毫秒。 */
    unsigned short lastmileDelay;
    /** 连接时长。 */
    int connectTimeMs;
    
    /** 系统CPU使用率。 */
    int systemCpuRate;
    /** RTC进程CPU使用率。 */
    int appCpuRate;
    
} RtcEngineStats;

/**
 * @ingroup CPP_DingRtcDef
 * @since 3.0
 * @brief 本地视频统计信息。
 */
struct LocalVideoStats {
    /** 视频流类型。 */
    RtcEngineVideoTrack track;
    /** 目标编码码率。 */
    unsigned int targetEncodeBitrate = 0;
    /** 实际编码码率。 */
    unsigned int actualEncodeBitrate = 0;
    /** 发送码率。 */
    unsigned int sentBitrate = 0;
    /** 发送帧率。 */
    unsigned int sentFps = 0;
    /** 编码器输出帧率。 */
    unsigned int encodeFps = 0;
    /** 采集帧率。 */
    unsigned int captureFps = 0;
    /** 每秒平均QP。 */
    unsigned int avgQpPerSec = 0;
    /** 编码帧宽。 */
    int encoderFrameWidth = 0;
    /** 编码帧高。 */
    int encoderFrameHeight = 0;
    /** 采集帧宽。 */
    int captureFrameWidth = 0;
    /** 采集帧高 */
    int captureFrameHeight = 0;
};

/**
 * @ingroup CPP_DingRtcDef
 * @since 3.0
 * @brief 远端视频统计信息。
 */
struct RemoteVideoStats {
    /** 用户ID。 */
    ding::rtc::String userId;
    /** 视频流类型。 */
    RtcEngineVideoTrack track;
    /** 视频帧宽。 */
    int width = 0;
    /** 视频帧高。 */
    int height = 0;
    /** 接收码率。 */
    int recvBitrate;
    /** 解码帧率。 */
    int decoderOutputFrameRate;
    /** 渲染帧率。 */
    int rendererOutputFrameRate;
    /** 丢包率。 */
    int packetLossRate;
};

/**
 * @ingroup CPP_DingRtcDef
 * @since 3.0
 * @brief 本地音频统计信息。
 */
struct LocalAudioStats {
    /** 音频流类型。 */
    RtcEngineAudioTrack track;
    /** 发送码率。 */
    unsigned int sentBitrate = 0;
    /** 发送的采样率。 */
    unsigned int sentSamplerate = 0;
    /** 声道数。 */
    unsigned int numChannel = 0;
    /** 输入音量。 */
    unsigned int inputLevel = 0;
};

/**
 * @ingroup CPP_DingRtcDef
 * @since 3.0
 * @brief 音频音量信息。
*/
struct AudioVolumeInfo {
    /** 用户ID（从AppServer下发的唯一标示符）。 */
    ding::rtc::String userId;
    /** 用户音量，取值范围：[0,255]。 */
    unsigned int volume = 0;
    /** 语音状态，取值：
     * 0：当前未说话。
     * 1：当前在说话。
     */
    unsigned int speechState = 0;
};

/**
 * @ingroup CPP_DingRtcDef
 * @since 3.0
 * @brief 远端音频统计信息。
 */
struct RemoteAudioStats {
    /** 远端用户UID。 */
    ding::rtc::String userId; 
    /** 音频丢包率。 */
    int packetLossRate = 0;
    /** 接收码率。 */
    int recvBitrate = 0;
    /** 卡顿次数。 */
    int totalFrozenTimes = 0;
};

/**
 * @ingroup CPP_DingRtcDef
 * @since 3.0
 * @brief 音频编码模式。
*/
typedef enum {
    /** 标准音质模式，默认16000Hz采样率，单声道，最大编码码率24kbps。 */              
    RtcEngineBasicQualityMode = 0x0000,
    /** 高音质模式，默认48000Hz采样率，单声道，最大编码码率64kbps。 */             
    RtcEngineHighQualityMode = 0x0001,
    /** 超高音质模式，默认48000Hz采样率，单声道，最大编码码率96kbps。 */        
    RtcEngineSuperHighQualityMode = 0x0002,
    // /** 立体声高音质模式，默认48000Hz采样率，双声道，最大编码码率80kbps。 */              
    // RtcEngineStereoHighQualityMode = 0x0003,
    // /** 立体声超高音质模式，默认48000Hz采样率，双声道，最大编码码率128kbps。 */        
    // RtcEngineStereoSuperHighQualityMode = 0x0004,   
} RtcEngineAudioProfile;

/**
 * @ingroup CPP_DingRtcDef
 * @since 3.0
 * @brief 音频场景模式。
 */
typedef enum {
    /** 默认场景，一般的音视频通信场景推荐使用。 */  
    RtcEngineSceneDefaultMode = 0x0000,
    // /** 音乐场景，高音质适用于音乐为主的场景。 */
    // RtcEngineSceneMusicMode = 0x0001,
} RtcEngineAudioScenario;


typedef enum {
    /** 采集的音频数据。 */
    RtcEngineAudioSourceCaptured = 0,
    /** 3A后的音频数据。 */
    RtcEngineAudioSourceProcessCaptured = 1,
    /** 推流的音频数据。 */
    RtcEngineAudioSourcePub = 2,
    /** 播放的音频数据。 */
    RtcEngineAudioSourcePlayback = 3,
} RtcEngineAudioSource;

/**
 * @ingroup CPP_DingRtcDef
 * @since 3.0
 * @brief 质量报告数据。
*/
typedef struct tagQualityReport {
    // cpu
    int cpu_physical_cores;
    int cpu_logic_cores;
    int cpu_frequency_mhz;
    int cpu_used_by_app; // percentage
    int cpu_used_by_other_apps; // percentage

    // memory
    int memory_total_mb;
    int memory_used_by_app;
    int memory_used_by_other_apps;

    // network
    /** 丢包率。 */
    int loss_rate; // percentage
    /** 延时。 */
    int rtt_ms;
    /** 接收码率。 */
    int recv_bitrate;
    /** 发送码率。 */
    int send_bitrate;
    int signal_level;
    /** 网络类型。 */
    char network_type[32];

    // audio up
    /** 音频发送码率。 */
    int audio_send_bitrate;
    /** 音频发送音量。 */
    int audio_send_volume;

    // audio down
    /** 音频接收码率。 */
    int audio_recv_bitrate;
    /** 音频接收音量。 */
    int audio_recv_volume; // in db

    // video up
    /** 视频发送码率。 */
    int camera_send_bitrate;
    /** 视频发送的采集帧率。 */
    int camera_send_capture_fps;
    int camera_send_drop_fps;
    /** 视频发送帧率。 */
    int camera_send_fps;
    int camera_send_hw;
    /** 视频发送宽度。 */
    int camera_send_width;
    /** 视频发送高度。 */
    int camera_send_height;

    /** 屏幕共享发送码率。 */
    int screenshare_send_bitrate;
    /** 屏幕共享采集帧率。 */
    int screenshare_send_capture_fps;
    int screenshare_send_drop_fps;
    /** 屏幕共享发送帧率。 */
    int screenshare_send_fps;
    int screenshare_send_hw;
    /** 屏幕共享宽度。 */
    int screenshare_send_width;
    /** 屏幕共享高度。 */
    int screenshare_send_height;

    // video down
    /** 视频接收码率。 */
    int camera_recv_bitrate;
    /** 视频接收帧率。 */
    int camera_recv_fps;
    /** 视频渲染器帧率。 */
    int camera_recv_render_fps;
    int camera_recv_hw;
    /** 视频接收宽度。 */
    int camera_recv_width;
    /** 视频接收高度。 */
    int camera_recv_height;

    /** 屏幕共享接收码率。 */
    int screenshare_recv_bitrate;
    /** 屏幕共享接收帧率。 */
    int screenshare_recv_fps;
    /** 屏幕共享渲染帧率。 */
    int screenshare_recv_render_fps;
    int screenshare_recv_hw;
    /** 屏幕共享接收宽度。 */
    int screenshare_recv_width;
    /** 屏幕共享接收高度。 */
    int screenshare_recv_height;
} QualityReport;

typedef enum {
    RtcEngineRecordModeAudioOnly = 0,  // record audio, generate *.opus file
    RtcEngineRecordModeVideoOnly,      // record video, generate *.mp4
    RtcEngineRecordModeAudioVideo,     // record audio & video, generate *.opus & *.mp4
} RtcEngineRecordMode;

typedef struct  {
  unsigned int width; 
  unsigned int height;
} RtcEngineRecordResolution;

struct RtcEngineRecordOptions{
    RtcEngineRecordOptions()
      : targetResolution({1280, 720}),
        recordMode(RtcEngineRecordModeVideoOnly),
        videoFps(20),
        useHardwareAccelerate(false) {}

    RtcEngineRecordResolution targetResolution;
    RtcEngineRecordMode recordMode;
    int videoFps;
    bool useHardwareAccelerate;
};

typedef enum {
    RtcEngineRecordStatusCompleted,  // record complete, generated .mp4 & .ogg file
    RtcEngineRecordStatusBuffered,   // mixer complete, generated .mcs file, need to transcode
    RtcEngineRecordStatusStoped,     // record stopped unexpectly, caused by internal error
    RtcEngineRecordStatusWarning,    // report an internal warning, continue record
} RtcEngineRecordStatusCode;

struct RtcEngineRecordBackgroundColor {
    uint8_t r;
    uint8_t g;
    uint8_t b;
};

struct RtcEngineRecordRational {
    uint32_t numerator;
    uint32_t denominator;
};

typedef enum {
    GRID, 
    SPEAKER,
    UNIQUE,
    STANDARD
} RtcEngineRecordLayoutMode;

struct RtcEngineRecordRegion {
    const char *uid;
    RtcEngineVideoTrack track;
};

class DING_RTC_API RtcEngineRecordRegionArray {
public:
    RtcEngineRecordRegionArray();
    virtual ~RtcEngineRecordRegionArray();
    RtcEngineRecordRegionArray(const RtcEngineRecordRegionArray &other);
    RtcEngineRecordRegionArray & operator=(const RtcEngineRecordRegionArray& other);

    void add(const RtcEngineRecordRegion &item);
    const RtcEngineRecordRegion &at(int index) const;
    void clear();
    int size() const;

private:
    void *data_{ nullptr };
};

/*
 *   Grid layout, Regions is sorted from up to
 *   down, and from left to right
 *   |=============|=============|
 *   |             |             |
 *   |      1      |      2      |
 *   |             |             |
 *   |=============|=============|
 *   |             |             |
 *   |      3      |      4      |
 *   |             |             |
 *   |=============|=============|
 */

/*
 *   Speaker layout, Regions is sorted as below
 *   |===================|=======|
 *   |                   |   2   |
 *   |                   |=======|
 *   |                   |   3   |
 *   |         1         |=======|
 *   |                   |   4   |
 *   |                   |=======|
 *   |                   |   5   |
 *   |===================|=======|
 *
 */

/*
 *   Standard layout:
 *   |======|======|======|======|
 *   |   2  |   3  |  4   |  5   |
 *   |===========================|
 *   |                           |
 *   |                           |
 *   |             1             |
 *   |                           |
 *   |                           |
 *   |                           |
 *   |===========================|
 */

struct RtcEngineRecordVideoLayout {
    RtcEngineRecordVideoLayout() : mode(GRID), backColor({0xff, 0xff, 0xff}) {}
    RtcEngineRecordLayoutMode mode;
    RtcEngineRecordRegionArray shapes;
    RtcEngineRecordBackgroundColor backColor;
};

//! @}

} // namespace rtc
} // namespace ding

#endif /* __DING_RTC_ENGINE_TYPES_H__ */
