﻿#ifndef __DING_RTC_ENGINE_UTILS_H__
#define __DING_RTC_ENGINE_UTILS_H__

#include "engine_conf.h"

#ifdef ALI_OS_WIN
# if defined DINGRTCSDK_EXPORTS
#  define DING_RTC_API __declspec(dllexport)
# else
#  define DING_RTC_API __declspec(dllimport)
# endif
#else
# if defined DINGRTCSDK_EXPORTS
#  define DING_RTC_API __attribute__ ((visibility ("default")))
# else
#  define DING_RTC_API
# endif
#endif

namespace ding {
namespace rtc {

//! \addtogroup Basic_Datatypes  数据类型参考
//! @{

class DING_RTC_API String
{
public:
    String(const char* str = 0);
    String(const String &other);
    String & operator=(const String& other);
    String & operator=(const char *str);
    String operator+(const String &other) const;
    String & operator+=(const String& other);
    virtual ~String(void);
    bool operator==(const String&) const;
    char& operator[](unsigned int);

    const char *c_str() const;
    bool isEmpty() const;
    int size() const;

private:
    char *data; // always != NULL
    int dataLen; // always > 0; including trailing '\0'
};

class DING_RTC_API StringArray
{
public:
    StringArray();
    virtual ~StringArray();
    StringArray(const StringArray &other);
    StringArray & operator=(const StringArray& other);

    void addString(const String &s);
    void clear();
    int size() const;
    String at(int index) const;

private:
    void *data; // always != NULL
};

class DING_RTC_API Dictionary
{
public:
    Dictionary();
    virtual ~Dictionary();
    Dictionary(const Dictionary &other);
    Dictionary & operator=(const Dictionary& other);

    void setStringValue(const char *key, const char  *val);
    void setBooleanValue(const char *key, bool val);
    String getStringValue(const char *key);
    bool getBooleanValue(const char *key, bool defVal);
    StringArray keys() const;

private:
    void *data; // always != NULL
};

//! @}

} // namespace rtc
} // namespace ding

#endif /* __DING_RTC_ENGINE_UTILS_H__ */
